class IKeyFrame:
 """ An System.Windows.Media.Animation.IKeyFrame interface implementation provides un-typed access to System.Windows.Media.Animation.KeyTime properties. """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 KeyTime=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets System.Windows.Media.Animation.IKeyFrame.KeyTime values associated with a KeyFrame object.



Get: KeyTime(self: IKeyFrame) -> KeyTime



Set: KeyTime(self: IKeyFrame)=value

"""

 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the value associated with a System.Windows.Media.Animation.KeyTime instance.



Get: Value(self: IKeyFrame) -> object



Set: Value(self: IKeyFrame)=value

"""





class IconConverter(ExpandableObjectConverter):
 """
 Converts an System.Drawing.Icon object from one data type to another. Access this class through the System.ComponentModel.TypeDescriptor object.

 

 IconConverter()
 """
 def CanConvertFrom(self,*__args):
  """
  CanConvertFrom(self: IconConverter,context: ITypeDescriptorContext,sourceType: Type) -> bool

  

   Determines whether this System.Drawing.IconConverter can convert an instance of a specified type 

    to an System.Drawing.Icon,using the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   sourceType: A System.Type that specifies the type you want to convert from.

   Returns: This method returns true if this System.Drawing.IconConverter can perform the conversion; 

    otherwise,false.
  """
  pass
 def CanConvertTo(self,*__args):
  """
  CanConvertTo(self: IconConverter,context: ITypeDescriptorContext,destinationType: Type) -> bool

  

   Determines whether this System.Drawing.IconConverter can convert an System.Drawing.Icon to an 

    instance of a specified type,using the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   destinationType: A System.Type that specifies the type you want to convert to.

   Returns: This method returns true if this System.Drawing.IconConverter can perform the conversion; 

    otherwise,false.
  """
  pass
 def ConvertFrom(self,*__args):
  """
  ConvertFrom(self: IconConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> object

  

   Converts a specified object to an System.Drawing.Icon.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: A System.Globalization.CultureInfo that holds information about a specific culture.

   value: The System.Object to be converted.

   Returns: If this method succeeds,it returns the System.Drawing.Icon that it created by converting the 

    specified object. Otherwise,it throws an exception.
  """
  pass
 def ConvertTo(self,*__args):
  """
  ConvertTo(self: IconConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object,destinationType: Type) -> object

  

   Converts an System.Drawing.Icon (or an object that can be cast to an System.Drawing.Icon) to a 

    specified type.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: A System.Globalization.CultureInfo object that specifies formatting conventions used by a 

    particular culture.

  

   value: The object to convert. This object should be of type icon or some type that can be cast to 

    System.Drawing.Icon.

  

   destinationType: The type to convert the icon to.

   Returns: This method returns the converted object.
  """
  pass



"""
Reference Wrappers

>>> pick = rpw.ui.Pick()
>>> references = pick.pick_element(multiple=True)
>>> references
[<rpw:Reference>, <rpw:Reference>]
>>> references[0].as_global_pt
<rpw:XYZ>

Linked Element

>>> reference = pick.pick_linked_element()
>>> element = reference.get_element()

"""

import rpw
from rpw import revit, DB
from rpw.db.element import Element
from rpw.db.xyz import XYZ
from rpw.utils.logger import logger
# from rpw.db.builtins import BipEnum


class Reference(Element):
    """
    `DB.Reference` Wrapper
    Inherits from :any:`Element`

    >>>

    Attribute:
        _revit_object (DB.Reference): Wrapped ``DB.Reference``
        doc (Document): Element Document
    """

    _revit_object_class = DB.Reference

    def __init__(self, reference, linked=False):
        if not linked:
            doc = revit.doc
        else:
            link_instance = revit.doc.GetElement(reference.ElementId)
            doc = link_instance.GetLinkDocument()

        super(Reference, self).__init__(reference, doc=doc)
        self.doc = doc
        self.linked = linked

    def __repr__(self):
        return super(Reference, self).__repr__(data={'id': self.id})

    @property
    def as_global_pt(self):
        """ Returns ``GlobalPoint`` property of Reference """
        pt = self._revit_object.GlobalPoint
        if pt:
            return XYZ(pt)

    @property
    def as_uv_pt(self):
        """ Returns ``UVPoint`` property of Reference - Face references only """
        pt = self._revit_object.UVPoint
        if pt:
            # TODO XYZ needs to handle XYZ
            return pt
            # return XYZ(pt)

    @property
    def id(self):
        """ ElementId of Reference """
        return self._revit_object.ElementId if not self.linked else self._revit_object.LinkedElementId

    def get_element(self, wrapped=True):
        """ Element of Reference """
        element = self.doc.GetElement(self.id)
        return element if not wrapped else Element(element)

    def get_geometry(self):
        """ GeometryObject from Reference """
        ref = self._revit_object
        return self.doc.GetElement(ref).GetGeometryObjectFromReference(ref)



class MemberDescriptor(object):
 """ Represents a class member,such as a property or event. This is an abstract base class. """
 def CreateAttributeCollection(self,*args):
  """
  CreateAttributeCollection(self: MemberDescriptor) -> AttributeCollection

  

   Creates a collection of attributes using the array of attributes passed to the constructor.

   Returns: A new System.ComponentModel.AttributeCollection that contains the 

    System.ComponentModel.MemberDescriptor.AttributeArray attributes.
  """
  pass
 def Equals(self,obj):
  """
  Equals(self: MemberDescriptor,obj: object) -> bool

  

   Compares this instance to the given object to see if they are equivalent.

  

   obj: The object to compare to the current instance.

   Returns: true if equivalent; otherwise,false.
  """
  pass
 def FillAttributes(self,*args):
  """
  FillAttributes(self: MemberDescriptor,attributeList: IList)

   When overridden in a derived class,adds the attributes of the inheriting class to the specified 

    list of attributes in the parent class.

  

  

   attributeList: An System.Collections.IList that lists the attributes in the parent class. Initially,this is 

    empty.
  """
  pass
 def FindMethod(self,*args):
  """
  FindMethod(componentClass: Type,name: str,args: Array[Type],returnType: Type,publicOnly: bool) -> MethodInfo

  

   Finds the given method through reflection,with an option to search only public methods.

  

   componentClass: The component that contains the method.

   name: The name of the method to find.

   args: An array of parameters for the method,used to choose between overloaded methods.

   returnType: The type to return for the method.

   publicOnly: Whether to restrict search to public methods.

   Returns: A System.Reflection.MethodInfo that represents the method,or null if the method is not found.

  FindMethod(componentClass: Type,name: str,args: Array[Type],returnType: Type) -> MethodInfo

  

   Finds the given method through reflection,searching only for public methods.

  

   componentClass: The component that contains the method.

   name: The name of the method to find.

   args: An array of parameters for the method,used to choose between overloaded methods.

   returnType: The type to return for the method.

   Returns: A System.Reflection.MethodInfo that represents the method,or null if the method is not found.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: MemberDescriptor) -> int

  

   Returns the hash code for this instance.

   Returns: A hash code for the current System.ComponentModel.MemberDescriptor.
  """
  pass
 def GetInvocationTarget(self,*args):
  """
  GetInvocationTarget(self: MemberDescriptor,type: Type,instance: object) -> object

  

   Retrieves the object that should be used during invocation of members.

  

   type: The System.Type of the invocation target.

   instance: The potential invocation target.

   Returns: The object to be used during member invocations.
  """
  pass
 def GetInvokee(self,*args):
  """
  GetInvokee(componentClass: Type,component: object) -> object

  

   Gets the component on which to invoke a method.

  

   componentClass: A System.Type representing the type of component this System.ComponentModel.MemberDescriptor is 

    bound to. For example,if this System.ComponentModel.MemberDescriptor describes a property,this 

    parameter should be the class that the property is declared on.

  

   component: An instance of the object to call.

   Returns: An instance of the component to invoke. This method returns a visual designer when the property 

    is attached to a visual designer.
  """
  pass
 def GetSite(self,*args):
  """
  GetSite(component: object) -> ISite

  

   Gets a component site for the given component.

  

   component: The component for which you want to find a site.

   Returns: The site of the component,or null if a site does not exist.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """
  __new__(cls: type,name: str)

  __new__(cls: type,name: str,attributes: Array[Attribute])

  __new__(cls: type,descr: MemberDescriptor)

  __new__(cls: type,oldMemberDescriptor: MemberDescriptor,newAttributes: Array[Attribute])
  """
  pass
 def __ne__(self,*args):
  pass
 AttributeArray=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets an array of attributes.



"""

 Attributes=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the collection of attributes for this member.



Get: Attributes(self: MemberDescriptor) -> AttributeCollection



"""

 Category=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the category to which the member belongs,as specified in the System.ComponentModel.CategoryAttribute.



Get: Category(self: MemberDescriptor) -> str



"""

 Description=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the description of the member,as specified in the System.ComponentModel.DescriptionAttribute.



Get: Description(self: MemberDescriptor) -> str



"""

 DesignTimeOnly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets whether this member should be set only at design time,as specified in the System.ComponentModel.DesignOnlyAttribute.



Get: DesignTimeOnly(self: MemberDescriptor) -> bool



"""

 DisplayName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name that can be displayed in a window,such as a Properties window.



Get: DisplayName(self: MemberDescriptor) -> str



"""

 IsBrowsable=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the member is browsable,as specified in the System.ComponentModel.BrowsableAttribute.



Get: IsBrowsable(self: MemberDescriptor) -> bool



"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the member.



Get: Name(self: MemberDescriptor) -> str



"""

 NameHashCode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the hash code for the name of the member,as specified in System.String.GetHashCode.



"""





class DataGridViewCellErrorTextNeededEventArgs(DataGridViewCellEventArgs):
 """ Provides data for the System.Windows.Forms.DataGridView.CellErrorTextNeeded event of a System.Windows.Forms.DataGridView control. """
 ErrorText=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the message that is displayed when the cell is selected.



Get: ErrorText(self: DataGridViewCellErrorTextNeededEventArgs) -> str



Set: ErrorText(self: DataGridViewCellErrorTextNeededEventArgs)=value

"""





class IRegistrationServices:
 """ Provides a set of services for registering and unregistering managed assemblies for use from COM. """
 def GetManagedCategoryGuid(self):
  """
  GetManagedCategoryGuid(self: IRegistrationServices) -> Guid

  

   Returns the GUID of the COM category that contains the managed classes.

   Returns: The GUID of the COM category that contains the managed classes.
  """
  pass
 def GetProgIdForType(self,type):
  """
  GetProgIdForType(self: IRegistrationServices,type: Type) -> str

  

   Retrieves the COM ProgID for a specified type.

  

   type: The type whose ProgID is being requested.

   Returns: The ProgID for the specified type.
  """
  pass
 def GetRegistrableTypesInAssembly(self,assembly):
  """
  GetRegistrableTypesInAssembly(self: IRegistrationServices,assembly: Assembly) -> Array[Type]

  

   Retrieves a list of classes in an assembly that would be registered by a call to 

    System.Runtime.InteropServices.IRegistrationServices.RegisterAssembly(System.Reflection.Assembly,

    System.Runtime.InteropServices.AssemblyRegistrationFlags).

  

  

   assembly: The assembly to search for classes.

   Returns: A System.Type array containing a list of classes in assembly.
  """
  pass
 def RegisterAssembly(self,assembly,flags):
  """
  RegisterAssembly(self: IRegistrationServices,assembly: Assembly,flags: AssemblyRegistrationFlags) -> bool

  

   Registers the classes in a managed assembly to enable creation from COM.

  

   assembly: The assembly to be registered.

   flags: An System.Runtime.InteropServices.AssemblyRegistrationFlags value indicating any special 

    settings needed when registering assembly.

  

   Returns: true if assembly contains types that were successfully registered; otherwise false if the 

    assembly contains no eligible types.
  """
  pass
 def RegisterTypeForComClients(self,type,g):
  """
  RegisterTypeForComClients(self: IRegistrationServices,type: Type,g: Guid) -> Guid

  

   Registers the specified type with COM using the specified GUID.

  

   type: The type to be registered for use from COM.

   g: GUID used to register the specified type.
  """
  pass
 def TypeRepresentsComType(self,type):
  """
  TypeRepresentsComType(self: IRegistrationServices,type: Type) -> bool

  

   Determines whether the specified type is a COM type.

  

   type: The type to determine if it is a COM type.

   Returns: true if the specified type is a COM type; otherwise false.
  """
  pass
 def TypeRequiresRegistration(self,type):
  """
  TypeRequiresRegistration(self: IRegistrationServices,type: Type) -> bool

  

   Determines whether the specified type requires registration.

  

   type: The type to check for COM registration requirements.

   Returns: true if the type must be registered for use from COM; otherwise false.
  """
  pass
 def UnregisterAssembly(self,assembly):
  """
  UnregisterAssembly(self: IRegistrationServices,assembly: Assembly) -> bool

  

   Unregisters the classes in a managed assembly.

  

   assembly: The assembly to be unregistered.

   Returns: true if assembly contains types that were successfully unregistered; otherwise false if the 

    assembly contains no eligible types.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



# encoding: utf-8
# module System.Net.Cache calls itself Cache
# from System,Version=4.0.0.0,Culture=neutral,PublicKeyToken=b77a5c561934e089
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class HttpCacheAgeControl(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the meaning of time values that control caching behavior for resources obtained using System.Net.HttpWebRequest objects.

 

 enum HttpCacheAgeControl,values: MaxAge (2),MaxAgeAndMaxStale (6),MaxAgeAndMinFresh (3),MaxStale (4),MinFresh (1),None (0)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 MaxAge=None
 MaxAgeAndMaxStale=None
 MaxAgeAndMinFresh=None
 MaxStale=None
 MinFresh=None
 None=None
 value__=None


class HttpRequestCacheLevel(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies caching behavior for resources obtained using the Hypertext Transfer protocol (HTTP).

 

 enum HttpRequestCacheLevel,values: BypassCache (1),CacheIfAvailable (3),CacheOnly (2),CacheOrNextCacheOnly (7),Default (0),NoCacheNoStore (6),Refresh (8),Reload (5),Revalidate (4)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 BypassCache=None
 CacheIfAvailable=None
 CacheOnly=None
 CacheOrNextCacheOnly=None
 Default=None
 NoCacheNoStore=None
 Refresh=None
 Reload=None
 Revalidate=None
 value__=None


class RequestCachePolicy(object):
 """
 Defines an application's caching requirements for resources obtained by using System.Net.WebRequest objects.

 

 RequestCachePolicy(level: RequestCacheLevel)

 RequestCachePolicy()
 """
 def ToString(self):
  """
  ToString(self: RequestCachePolicy) -> str

  

   Returns a string representation of this instance.

   Returns: A System.String containing the System.Net.Cache.RequestCachePolicy.Level for this instance.
  """
  pass
 @staticmethod
 def __new__(self,level=None):
  """
  __new__(cls: type)

  __new__(cls: type,level: RequestCacheLevel)
  """
  pass
 Level=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Net.Cache.RequestCacheLevel value specified when this instance was constructed.



Get: Level(self: RequestCachePolicy) -> RequestCacheLevel



"""



class HttpRequestCachePolicy(RequestCachePolicy):
 """
 Defines an application's caching requirements for resources obtained by using System.Net.HttpWebRequest objects.

 

 HttpRequestCachePolicy()

 HttpRequestCachePolicy(level: HttpRequestCacheLevel)

 HttpRequestCachePolicy(cacheAgeControl: HttpCacheAgeControl,ageOrFreshOrStale: TimeSpan)

 HttpRequestCachePolicy(cacheAgeControl: HttpCacheAgeControl,maxAge: TimeSpan,freshOrStale: TimeSpan)

 HttpRequestCachePolicy(cacheSyncDate: DateTime)

 HttpRequestCachePolicy(cacheAgeControl: HttpCacheAgeControl,maxAge: TimeSpan,freshOrStale: TimeSpan,cacheSyncDate: DateTime)
 """
 def ToString(self):
  """
  ToString(self: HttpRequestCachePolicy) -> str

  

   Returns a string representation of this instance.

   Returns: A System.String value that contains the property values for this instance.
  """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type)

  __new__(cls: type,level: HttpRequestCacheLevel)

  __new__(cls: type,cacheAgeControl: HttpCacheAgeControl,ageOrFreshOrStale: TimeSpan)

  __new__(cls: type,cacheAgeControl: HttpCacheAgeControl,maxAge: TimeSpan,freshOrStale: TimeSpan)

  __new__(cls: type,cacheSyncDate: DateTime)

  __new__(cls: type,cacheAgeControl: HttpCacheAgeControl,maxAge: TimeSpan,freshOrStale: TimeSpan,cacheSyncDate: DateTime)
  """
  pass
 CacheSyncDate=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the cache synchronization date for this instance.



Get: CacheSyncDate(self: HttpRequestCachePolicy) -> DateTime



"""

 Level=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Net.Cache.HttpRequestCacheLevel value that was specified when this instance was created.



Get: Level(self: HttpRequestCachePolicy) -> HttpRequestCacheLevel



"""

 MaxAge=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the maximum age permitted for a resource returned from the cache.



Get: MaxAge(self: HttpRequestCachePolicy) -> TimeSpan



"""

 MaxStale=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the maximum staleness value that is permitted for a resource returned from the cache.



Get: MaxStale(self: HttpRequestCachePolicy) -> TimeSpan



"""

 MinFresh=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the minimum freshness that is permitted for a resource returned from the cache.



Get: MinFresh(self: HttpRequestCachePolicy) -> TimeSpan



"""



class RequestCacheLevel(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies caching behavior for resources obtained using System.Net.WebRequest and its derived classes.

 

 enum RequestCacheLevel,values: BypassCache (1),CacheIfAvailable (3),CacheOnly (2),Default (0),NoCacheNoStore (6),Reload (5),Revalidate (4)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 BypassCache=None
 CacheIfAvailable=None
 CacheOnly=None
 Default=None
 NoCacheNoStore=None
 Reload=None
 Revalidate=None
 value__=None





"""Charts engine for output window."""
# pylint: disable=C0103
from json import JSONEncoder

from pyrevit.coreutils import timestamp, random_rgba_color

# CHARTS_ENGINE = 'Chart.js'
CHARTS_ENGINE = 'Chart.bundle.js'

# chart.js chart types
LINE_CHART = 'line'
BAR_CHART = 'bar'
RADAR_CHART = 'radar'
POLAR_CHART = 'polarArea'
PIE_CHART = 'pie'
DOUGHNUT_CHART = 'doughnut'
BUBBLE_CHART = 'bubble'


CHARTS_JS_PATH = \
    "https://cdnjs.cloudflare.com/ajax/libs/Chart.js/{version}/Chart.min.js"


SCRIPT_TEMPLATE = \
    "var ctx = document.getElementById('{canvas_id}').getContext('2d');" \
    "var chart = new Chart(ctx, {canvas_code});"


class _ChartsDataSetEncode(JSONEncoder):
    """JSON encoder for chart data sets."""
    def default(self, dataset_obj):  # pylint: disable=E0202, W0221
        data_dict = dataset_obj.__dict__.copy()
        for key, value in data_dict.items():
            if key.startswith('_') or value == '' or value == []:
                data_dict.pop(key)

        return data_dict


class PyRevitOutputChartOptions(object):
    """Chart options wrapper object."""
    def __init__(self):
        pass


class PyRevitOutputChartDataset(object):
    """Chart dataset wrapper object."""
    def __init__(self, label):
        self.label = label
        self.data = []
        self.backgroundColor = ''

    def set_color(self, *args):
        """Set dataset color.

        Arguments are expected to be R, G, B, A values.

        Examples:
            ```python
            dataset_obj.set_color(0xFF, 0x8C, 0x8D, 0.8)
            ```
        """
        if len(args) == 4:
            self.backgroundColor = 'rgba({},{},{},{})'.format(args[0],
                                                              args[1],
                                                              args[2],
                                                              args[3])
        elif len(args) == 1:
            self.backgroundColor = '{}'.format(args[0])


class PyRevitOutputChartData(object):
    """Chart data wrapper object."""
    def __init__(self):
        self.labels = ''
        self.datasets = []

    def new_dataset(self, dataset_label):
        """Create new data set.

        Args:
            dataset_label (str): dataset label

        Returns:
            (PyRevitOutputChartDataset): dataset wrapper object

        Examples:
            ```python
            chart.data.new_dataset('set_a')
            ```
        """
        new_dataset = PyRevitOutputChartDataset(dataset_label)
        self.datasets.append(new_dataset)
        return new_dataset


class PyRevitOutputChart(object):
    """Chart wrapper object for output window.

    Attributes:
        output (pyrevit.output.PyRevitOutputWindow):
            output window wrapper object
        chart_type (str): chart type name
    """
    def __init__(self, output, chart_type=LINE_CHART, version=None):
        self._output = output
        self._style = None
        self._width = self._height = None
        self._version = version or '2.8.0'

        self.type = chart_type
        self.data = PyRevitOutputChartData()

        self.options = PyRevitOutputChartOptions()
        # # common chart options and their default values
        # chart.options.responsive = True
        # chart.options.responsiveAnimationDuration = 0
        # chart.options.maintainAspectRatio = True
        #
        # # layout options
        # chart.options.layout = {'padding': 0}
        #
        # # title options
        # # position:
        # # Position of the title. Possible values are 'top',
        # # 'left', 'bottom' and 'right'.
        # chart.options.title = {'display': False,
        #                        'position': 'top',
        #                        'fullWidth': True,
        #                        'fontSize': 12,
        #                        'fontFamily': 'Arial',
        #                        'fontColor': '#666',
        #                        'fontStyle': 'bold',
        #                        'padding': 10,
        #                        'text': ''
        #                        }
        #
        # # legend options
        # chart.options.legend = {'display': True,
        #                         'position': 'top',
        #                         'fullWidth': True,
        #                         'reverse': False,
        #                         'labels': {'boxWidth': 40,
        #                                    'fontSize': 12,
        #                                    'fontStyle': 'normal',
        #                                    'fontColor': '#666',
        #                                    'fontFamily': 'Arial',
        #                                    'padding': 10,
        #                                    'usePointStyle': True
        #                                    }
        #                         }
        #
        # # tooltips options
        # # intersect:
        # # if true, the tooltip mode applies only when the mouse
        # # position intersects with an element.
        # # If false, the mode will be applied at all times
        # chart.options.tooltips = {'enabled': True,
        #                           'intersect': True,
        #                           'backgroundColor': 'rgba(0,0,0,0.8)',
        #                           'caretSize': 5,
        #                           'displayColors': True}

    def _setup_charts(self):
        cur_head = self._output.get_head_html()
        charts_js_path = CHARTS_JS_PATH.format(version=self._version)
        if charts_js_path not in cur_head:
            self._output.inject_script('', {'src': charts_js_path})

    @staticmethod
    def _make_canvas_unique_id():
        return 'chart{}'.format(timestamp())

    def _make_canvas_code(self, canvas_id):
        attribs = ''
        attribs += ' id="{}"'.format(canvas_id)
        if self._style:
            attribs += ' style="{}"'.format(self._style)
        else:
            if self._width:
                attribs += ' width="{}px"'.format(self._width)
            if self._height:
                attribs += ' height="{}px"'.format(self._height)

        return '<canvas {}></canvas>'.format(attribs)

    def _make_charts_script(self, canvas_id):
        return SCRIPT_TEMPLATE.format(
            canvas_id=canvas_id,
            canvas_code=_ChartsDataSetEncode().encode(self))

    def randomize_colors(self):
        """Randomize chart datasets colors."""
        if self.type in [POLAR_CHART, PIE_CHART, DOUGHNUT_CHART]:
            for dataset in self.data.datasets:
                dataset.backgroundColor = [random_rgba_color()
                                           for _ in range(0, len(dataset.data))]
        else:
            for dataset in self.data.datasets:
                dataset.backgroundColor = random_rgba_color()

    def set_width(self, width):
        """Set chart width on output window."""
        self._width = width

    def set_height(self, height):
        """Set chart height on output window."""
        self._height = height

    def set_style(self, html_style):
        """Set chart styling.

        Args:
            html_style (str): inline html css styling string

        Examples:
            ```python
            chart.set_style('height:150px')
            ```
        """
        self._style = html_style

    def draw(self):
        """Request chart to draw itself on output window."""
        self._setup_charts()
        # setup canvas
        canvas_id = self._make_canvas_unique_id()
        canvas_code = self._make_canvas_code(canvas_id)
        self._output.print_html(canvas_code)
        # make the code
        js_code = self._make_charts_script(canvas_id)
        self._output.inject_script(js_code, body=True)



class DelimitedListTraceListener(TextWriterTraceListener,IDisposable):
 """
 Directs tracing or debugging output to a text writer,such as a stream writer,or to a stream,such as a file stream.

 

 DelimitedListTraceListener(stream: Stream)

 DelimitedListTraceListener(stream: Stream,name: str)

 DelimitedListTraceListener(writer: TextWriter)

 DelimitedListTraceListener(writer: TextWriter,name: str)

 DelimitedListTraceListener(fileName: str)

 DelimitedListTraceListener(fileName: str,name: str)
 """
 def Dispose(self):
  """
  Dispose(self: TextWriterTraceListener,disposing: bool)

   Disposes this System.Diagnostics.TextWriterTraceListener object.

  

   disposing: true to release managed resources; if false,

    System.Diagnostics.TextWriterTraceListener.Dispose(System.Boolean) has no effect.
  """
  pass
 def GetSupportedAttributes(self,*args):
  """
  GetSupportedAttributes(self: DelimitedListTraceListener) -> Array[str]

  

   Returns the custom configuration file attribute supported by the delimited trace listener.

   Returns: A string array that contains the single value "delimiter".
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def TraceData(self,eventCache,source,eventType,id,data):
  """
  TraceData(self: DelimitedListTraceListener,eventCache: TraceEventCache,source: str,eventType: TraceEventType,id: int,*data: Array[object])

   Writes trace information,an array of data objects,and event information to the output file or 

    stream.

  

  

   eventCache: A System.Diagnostics.TraceEventCache object that contains the current process ID,thread ID,and 

    stack trace information.

  

   source: A name used to identify the output,typically the name of the application that generated the 

    trace event.

  

   eventType: One of the System.Diagnostics.TraceEventType values specifying the type of event that has caused 

    the trace.

  

   id: A numeric identifier for the event.

   data: An array of data objects to write to the output file or stream.

  TraceData(self: DelimitedListTraceListener,eventCache: TraceEventCache,source: str,eventType: TraceEventType,id: int,data: object)

   Writes trace information,a data object,and event information to the output file or stream.

  

   eventCache: A System.Diagnostics.TraceEventCache object that contains the current process ID,thread ID,and 

    stack trace information.

  

   source: A name used to identify the output,typically the name of the application that generated the 

    trace event.

  

   eventType: One of the System.Diagnostics.TraceEventType values specifying the type of event that has caused 

    the trace.

  

   id: A numeric identifier for the event.

   data: A data object to write to the output file or stream.
  """
  pass
 def TraceEvent(self,eventCache,source,eventType,id,*__args):
  """
  TraceEvent(self: DelimitedListTraceListener,eventCache: TraceEventCache,source: str,eventType: TraceEventType,id: int,message: str)

   Writes trace information,a message,and event information to the output file or stream.

  

   eventCache: A System.Diagnostics.TraceEventCache object that contains the current process ID,thread ID,and 

    stack trace information.

  

   source: A name used to identify the output,typically the name of the application that generated the 

    trace event.

  

   eventType: One of the System.Diagnostics.TraceEventType values specifying the type of event that has caused 

    the trace.

  

   id: A numeric identifier for the event.

   message: The trace message to write to the output file or stream.

  TraceEvent(self: DelimitedListTraceListener,eventCache: TraceEventCache,source: str,eventType: TraceEventType,id: int,format: str,*args: Array[object])

   Writes trace information,a formatted array of objects,and event information to the output file 

    or stream.

  

  

   eventCache: A System.Diagnostics.TraceEventCache object that contains the current process ID,thread ID,and 

    stack trace information.

  

   source: A name used to identify the output,typically the name of the application that generated the 

    trace event.

  

   eventType: One of the System.Diagnostics.TraceEventType values specifying the type of event that has caused 

    the trace.

  

   id: A numeric identifier for the event.

   format: A format string that contains zero or more format items that correspond to objects in the args 

    array.

  

   args: An array containing zero or more objects to format.
  """
  pass
 def WriteIndent(self,*args):
  """
  WriteIndent(self: TraceListener)

   Writes the indent to the listener you create when you implement this class,and resets the 

    System.Diagnostics.TraceListener.NeedIndent property to false.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,stream: Stream)

  __new__(cls: type,stream: Stream,name: str)

  __new__(cls: type,writer: TextWriter)

  __new__(cls: type,writer: TextWriter,name: str)

  __new__(cls: type,fileName: str)

  __new__(cls: type,fileName: str,name: str)
  """
  pass
 Delimiter=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the delimiter for the delimited list.



Get: Delimiter(self: DelimitedListTraceListener) -> str



Set: Delimiter(self: DelimitedListTraceListener)=value

"""

 NeedIndent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether to indent the output.



"""





class DesignerAttribute(Attribute,_Attribute):
 """
 Specifies the class used to implement design-time services for a component.

 

 DesignerAttribute(designerTypeName: str)

 DesignerAttribute(designerType: Type)

 DesignerAttribute(designerTypeName: str,designerBaseTypeName: str)

 DesignerAttribute(designerTypeName: str,designerBaseType: Type)

 DesignerAttribute(designerType: Type,designerBaseType: Type)
 """
 def Equals(self,obj):
  """
  Equals(self: DesignerAttribute,obj: object) -> bool

  

   Returns whether the value of the given object is equal to the current 

    System.ComponentModel.DesignerAttribute.

  

  

   obj: The object to test the value equality of.

   Returns: true if the value of the given object is equal to that of the current; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """ GetHashCode(self: DesignerAttribute) -> int """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,designerTypeName: str)

  __new__(cls: type,designerType: Type)

  __new__(cls: type,designerTypeName: str,designerBaseTypeName: str)

  __new__(cls: type,designerTypeName: str,designerBaseType: Type)

  __new__(cls: type,designerType: Type,designerBaseType: Type)
  """
  pass
 def __ne__(self,*args):
  pass
 DesignerBaseTypeName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the base type of this designer.



Get: DesignerBaseTypeName(self: DesignerAttribute) -> str



"""

 DesignerTypeName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the designer type associated with this designer attribute.



Get: DesignerTypeName(self: DesignerAttribute) -> str



"""

 TypeId=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a unique ID for this attribute type.



Get: TypeId(self: DesignerAttribute) -> object



"""





class ComDefaultInterfaceAttribute(Attribute,_Attribute):
 """
 Specifies a default interface to expose to COM. This class cannot be inherited.

 

 ComDefaultInterfaceAttribute(defaultInterface: Type)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,defaultInterface):
  """ __new__(cls: type,defaultInterface: Type) """
  pass
 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Type object that specifies the default interface to expose to COM.



Get: Value(self: ComDefaultInterfaceAttribute) -> Type



"""





class CharEnumerator(object,IEnumerator,ICloneable,IEnumerator[Char],IDisposable):
 """ Supports iterating over a System.String object and reading its individual characters. This class cannot be inherited. """
 def Clone(self):
  """
  Clone(self: CharEnumerator) -> object

  

   Creates a copy of the current System.CharEnumerator object.

   Returns: An System.Object that is a copy of the current System.CharEnumerator object.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: CharEnumerator)

   Releases all resources used by the current instance of the System.CharEnumerator class.
  """
  pass
 def MoveNext(self):
  """
  MoveNext(self: CharEnumerator) -> bool

  

   Increments the internal index of the current System.CharEnumerator object to the next character 

    of the enumerated string.

  

   Returns: true if the index is successfully incremented and within the enumerated string; otherwise,false.
  """
  pass
 def next(self,*args):
  """ next(self: object) -> object """
  pass
 def Reset(self):
  """
  Reset(self: CharEnumerator)

   Initializes the index to a position logically before the first character of the enumerated 

    string.
  """
  pass
 def __contains__(self,*args):
  """ __contains__[Char](enumerator: IEnumerator[Char],value: Char) -> bool """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerator) -> object """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 Current=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the currently referenced character in the string enumerated by this System.CharEnumerator object.



Get: Current(self: CharEnumerator) -> Char



"""





class ToolStripItemTextRenderEventArgs(ToolStripItemRenderEventArgs):
 """
 Provides data for the System.Windows.Forms.ToolStripRenderer.RenderItemText event.

 

 ToolStripItemTextRenderEventArgs(g: Graphics,item: ToolStripItem,text: str,textRectangle: Rectangle,textColor: Color,textFont: Font,format: TextFormatFlags)

 ToolStripItemTextRenderEventArgs(g: Graphics,item: ToolStripItem,text: str,textRectangle: Rectangle,textColor: Color,textFont: Font,textAlign: ContentAlignment)
 """
 @staticmethod
 def __new__(self,g,item,text,textRectangle,textColor,textFont,*__args):
  """
  __new__(cls: type,g: Graphics,item: ToolStripItem,text: str,textRectangle: Rectangle,textColor: Color,textFont: Font,format: TextFormatFlags)

  __new__(cls: type,g: Graphics,item: ToolStripItem,text: str,textRectangle: Rectangle,textColor: Color,textFont: Font,textAlign: ContentAlignment)
  """
  pass
 Text=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the text to be drawn on the System.Windows.Forms.ToolStripItem.



Get: Text(self: ToolStripItemTextRenderEventArgs) -> str



Set: Text(self: ToolStripItemTextRenderEventArgs)=value

"""

 TextColor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the color of the System.Windows.Forms.ToolStripItem text.



Get: TextColor(self: ToolStripItemTextRenderEventArgs) -> Color



Set: TextColor(self: ToolStripItemTextRenderEventArgs)=value

"""

 TextDirection=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets whether the text is drawn vertically or horizontally.



Get: TextDirection(self: ToolStripItemTextRenderEventArgs) -> ToolStripTextDirection



Set: TextDirection(self: ToolStripItemTextRenderEventArgs)=value

"""

 TextFont=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the font of the text drawn on the System.Windows.Forms.ToolStripItem.



Get: TextFont(self: ToolStripItemTextRenderEventArgs) -> Font



Set: TextFont(self: ToolStripItemTextRenderEventArgs)=value

"""

 TextFormat=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the display and layout information of the text drawn on the System.Windows.Forms.ToolStripItem.



Get: TextFormat(self: ToolStripItemTextRenderEventArgs) -> TextFormatFlags



Set: TextFormat(self: ToolStripItemTextRenderEventArgs)=value

"""

 TextRectangle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the rectangle that represents the bounds to draw the text in.



Get: TextRectangle(self: ToolStripItemTextRenderEventArgs) -> Rectangle



Set: TextRectangle(self: ToolStripItemTextRenderEventArgs)=value

"""





class BindingManagerBase(object):
 """
 Manages all System.Windows.Forms.Binding objects that are bound to the same data source and data member. This class is abstract.

 

 BindingManagerBase()
 """
 def AddNew(self):
  """
  AddNew(self: BindingManagerBase)

   When overridden in a derived class,adds a new item to the underlying list.
  """
  pass
 def CancelCurrentEdit(self):
  """
  CancelCurrentEdit(self: BindingManagerBase)

   When overridden in a derived class,cancels the current edit.
  """
  pass
 def EndCurrentEdit(self):
  """
  EndCurrentEdit(self: BindingManagerBase)

   When overridden in a derived class,ends the current edit.
  """
  pass
 def GetItemProperties(self):
  """
  GetItemProperties(self: BindingManagerBase) -> PropertyDescriptorCollection

  

   When overridden in a derived class,gets the collection of property descriptors for the binding.

   Returns: A System.ComponentModel.PropertyDescriptorCollection that represents the property descriptors 

    for the binding.
  """
  pass
 def GetListName(self,*args):
  """
  GetListName(self: BindingManagerBase,listAccessors: ArrayList) -> str

  

   When overridden in a derived class,gets the name of the list supplying the data for the binding.

  

   listAccessors: An System.Collections.ArrayList containing the table's bound properties.

   Returns: The name of the list supplying the data for the binding.
  """
  pass
 def OnBindingComplete(self,*args):
  """
  OnBindingComplete(self: BindingManagerBase,args: BindingCompleteEventArgs)

   Raises the System.Windows.Forms.BindingManagerBase.BindingComplete event.

  

   args: A System.Windows.Forms.BindingCompleteEventArgs  that contains the event data.
  """
  pass
 def OnCurrentChanged(self,*args):
  """
  OnCurrentChanged(self: BindingManagerBase,e: EventArgs)

   Raises the System.Windows.Forms.BindingManagerBase.CurrentChanged event.

  

   e: The System.EventArgs that contains the event data.
  """
  pass
 def OnCurrentItemChanged(self,*args):
  """
  OnCurrentItemChanged(self: BindingManagerBase,e: EventArgs)

   Raises the System.Windows.Forms.BindingManagerBase.CurrentItemChanged event.

  

   e: The System.EventArgs that contains the event data.
  """
  pass
 def OnDataError(self,*args):
  """
  OnDataError(self: BindingManagerBase,e: Exception)

   Raises the System.Windows.Forms.BindingManagerBase.DataError event.

  

   e: An System.Exception that caused the System.Windows.Forms.BindingManagerBase.DataError event to 

    occur.
  """
  pass
 def PullData(self,*args):
  """
  PullData(self: BindingManagerBase)

   Pulls data from the data-bound control into the data source,returning no information.
  """
  pass
 def PushData(self,*args):
  """
  PushData(self: BindingManagerBase)

   Pushes data from the data source into the data-bound control,returning no information.
  """
  pass
 def RemoveAt(self,index):
  """
  RemoveAt(self: BindingManagerBase,index: int)

   When overridden in a derived class,deletes the row at the specified index from the underlying 

    list.

  

  

   index: The index of the row to delete.
  """
  pass
 def ResumeBinding(self):
  """
  ResumeBinding(self: BindingManagerBase)

   When overridden in a derived class,resumes data binding.
  """
  pass
 def SuspendBinding(self):
  """
  SuspendBinding(self: BindingManagerBase)

   When overridden in a derived class,suspends data binding.
  """
  pass
 def UpdateIsBinding(self,*args):
  """
  UpdateIsBinding(self: BindingManagerBase)

   When overridden in a derived class,updates the binding.
  """
  pass
 Bindings=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the collection of bindings being managed.



Get: Bindings(self: BindingManagerBase) -> BindingsCollection



"""

 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets the number of rows managed by the System.Windows.Forms.BindingManagerBase.



Get: Count(self: BindingManagerBase) -> int



"""

 Current=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets the current object.



Get: Current(self: BindingManagerBase) -> object



"""

 IsBindingSuspended=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether binding is suspended.



Get: IsBindingSuspended(self: BindingManagerBase) -> bool



"""

 Position=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets or sets the position in the underlying list that controls bound to this data source point to.



Get: Position(self: BindingManagerBase) -> int



Set: Position(self: BindingManagerBase)=value

"""


 BindingComplete=None
 CurrentChanged=None
 CurrentItemChanged=None
 DataError=None
 onCurrentChangedHandler=None
 onPositionChangedHandler=None
 PositionChanged=None




# encoding: utf-8
# module System.Timers calls itself Timers
# from System,Version=4.0.0.0,Culture=neutral,PublicKeyToken=b77a5c561934e089
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class ElapsedEventArgs(EventArgs):
 """ Provides data for the System.Timers.Timer.Elapsed event. """
 SignalTime=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the time the System.Timers.Timer.Elapsed event was raised.



Get: SignalTime(self: ElapsedEventArgs) -> DateTime



"""



class ElapsedEventHandler(MulticastDelegate,ICloneable,ISerializable):
 """
 Represents the method that will handle the System.Timers.Timer.Elapsed event of a System.Timers.Timer.

 

 ElapsedEventHandler(object: object,method: IntPtr)
 """
 def BeginInvoke(self,sender,e,callback,object):
  """ BeginInvoke(self: ElapsedEventHandler,sender: object,e: ElapsedEventArgs,callback: AsyncCallback,object: object) -> IAsyncResult """
  pass
 def CombineImpl(self,*args):
  """
  CombineImpl(self: MulticastDelegate,follow: Delegate) -> Delegate

  

   Combines this System.Delegate with the specified System.Delegate to form a new delegate.

  

   follow: The delegate to combine with this delegate.

   Returns: A delegate that is the new root of the System.MulticastDelegate invocation list.
  """
  pass
 def DynamicInvokeImpl(self,*args):
  """
  DynamicInvokeImpl(self: Delegate,args: Array[object]) -> object

  

   Dynamically invokes (late-bound) the method represented by the current delegate.

  

   args: An array of objects that are the arguments to pass to the method represented by the current 

    delegate.-or- null,if the method represented by the current delegate does not require 

    arguments.

  

   Returns: The object returned by the method represented by the delegate.
  """
  pass
 def EndInvoke(self,result):
  """ EndInvoke(self: ElapsedEventHandler,result: IAsyncResult) """
  pass
 def GetMethodImpl(self,*args):
  """
  GetMethodImpl(self: MulticastDelegate) -> MethodInfo

  

   Returns a static method represented by the current System.MulticastDelegate.

   Returns: A static method represented by the current System.MulticastDelegate.
  """
  pass
 def Invoke(self,sender,e):
  """ Invoke(self: ElapsedEventHandler,sender: object,e: ElapsedEventArgs) """
  pass
 def RemoveImpl(self,*args):
  """
  RemoveImpl(self: MulticastDelegate,value: Delegate) -> Delegate

  

   Removes an element from the invocation list of this System.MulticastDelegate that is equal to 

    the specified delegate.

  

  

   value: The delegate to search for in the invocation list.

   Returns: If value is found in the invocation list for this instance,then a new System.Delegate without 

    value in its invocation list; otherwise,this instance with its original invocation list.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,object,method):
  """ __new__(cls: type,object: object,method: IntPtr) """
  pass
 def __reduce_ex__(self,*args):
  pass

class Timer(Component,IComponent,IDisposable,ISupportInitialize):
 """
 Generates recurring events in an application.

 

 Timer()

 Timer(interval: float)
 """
 def BeginInit(self):
  """
  BeginInit(self: Timer)

   Begins the run-time initialization of a System.Timers.Timer that is used on a form or by another 

    component.
  """
  pass
 def Close(self):
  """
  Close(self: Timer)

   Releases the resources used by the System.Timers.Timer.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: Timer,disposing: bool)

   Releases all resources used by the current System.Timers.Timer.

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def EndInit(self):
  """
  EndInit(self: Timer)

   Ends the run-time initialization of a System.Timers.Timer that is used on a form or by another 

    component.
  """
  pass
 def GetService(self,*args):
  """
  GetService(self: Component,service: Type) -> object

  

   Returns an object that represents a service provided by the System.ComponentModel.Component or 

    by its System.ComponentModel.Container.

  

  

   service: A service provided by the System.ComponentModel.Component.

   Returns: An System.Object that represents a service provided by the System.ComponentModel.Component,or 

    null if the System.ComponentModel.Component does not provide the specified service.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def Start(self):
  """
  Start(self: Timer)

   Starts raising the System.Timers.Timer.Elapsed event by setting System.Timers.Timer.Enabled to 

    true.
  """
  pass
 def Stop(self):
  """
  Stop(self: Timer)

   Stops raising the System.Timers.Timer.Elapsed event by setting System.Timers.Timer.Enabled to 

    false.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,interval=None):
  """
  __new__(cls: type)

  __new__(cls: type,interval: float)
  """
  pass
 def __str__(self,*args):
  pass
 AutoReset=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the System.Timers.Timer should raise the System.Timers.Timer.Elapsed event each time the specified interval elapses or only after the first time it elapses.



Get: AutoReset(self: Timer) -> bool



Set: AutoReset(self: Timer)=value

"""

 CanRaiseEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the component can raise an event.



"""

 DesignMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the System.ComponentModel.Component is currently in design mode.



"""

 Enabled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the System.Timers.Timer should raise the System.Timers.Timer.Elapsed event.



Get: Enabled(self: Timer) -> bool



Set: Enabled(self: Timer)=value

"""

 Events=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of event handlers that are attached to this System.ComponentModel.Component.



"""

 Interval=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the interval at which to raise the System.Timers.Timer.Elapsed event.



Get: Interval(self: Timer) -> float



Set: Interval(self: Timer)=value

"""

 Site=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the site that binds the System.Timers.Timer to its container in design mode.



Get: Site(self: Timer) -> ISite



Set: Site(self: Timer)=value

"""

 SynchronizingObject=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the object used to marshal event-handler calls that are issued when an interval has elapsed.



Get: SynchronizingObject(self: Timer) -> ISynchronizeInvoke



Set: SynchronizingObject(self: Timer)=value

"""


 Elapsed=None


class TimersDescriptionAttribute(DescriptionAttribute,_Attribute):
 """
 Sets the description that visual designers can display when referencing an event,extender,or property.

 

 TimersDescriptionAttribute(description: str)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,description):
  """ __new__(cls: type,description: str) """
  pass
 Description=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the description that visual designers can display when referencing an event,extender,or property.



Get: Description(self: TimersDescriptionAttribute) -> str



"""

 DescriptionValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the string stored as the description.



"""






class ComplexBindingPropertiesAttribute(Attribute,_Attribute):
 """
 Specifies the data source and data member properties for a component that supports complex data binding. This class cannot be inherited.

 

 ComplexBindingPropertiesAttribute()

 ComplexBindingPropertiesAttribute(dataSource: str)

 ComplexBindingPropertiesAttribute(dataSource: str,dataMember: str)
 """
 def Equals(self,obj):
  """
  Equals(self: ComplexBindingPropertiesAttribute,obj: object) -> bool

  

   Determines whether the specified System.Object is equal to the current 

    System.ComponentModel.ComplexBindingPropertiesAttribute instance.

  

  

   obj: The System.Object to compare with the current 

    System.ComponentModel.ComplexBindingPropertiesAttribute instance

  

   Returns: true if the object is equal to the current instance; otherwise,false,indicating they are not 

    equal.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: ComplexBindingPropertiesAttribute) -> int

   Returns: A 32-bit signed integer hash code.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,dataSource=None,dataMember=None):
  """
  __new__(cls: type)

  __new__(cls: type,dataSource: str)

  __new__(cls: type,dataSource: str,dataMember: str)
  """
  pass
 def __ne__(self,*args):
  pass
 DataMember=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the data member property for the component to which the System.ComponentModel.ComplexBindingPropertiesAttribute is bound.



Get: DataMember(self: ComplexBindingPropertiesAttribute) -> str



"""

 DataSource=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the data source property for the component to which the System.ComponentModel.ComplexBindingPropertiesAttribute is bound.



Get: DataSource(self: ComplexBindingPropertiesAttribute) -> str



"""


 Default=None




# encoding: utf-8
# module System.Diagnostics calls itself Diagnostics
# from mscorlib,Version=4.0.0.0,Culture=neutral,PublicKeyToken=b77a5c561934e089,System,Version=4.0.0.0,Culture=neutral,PublicKeyToken=b77a5c561934e089
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes
from __init___parts.Switch import Switch
from __init___parts.BooleanSwitch import BooleanSwitch
from __init___parts.ConditionalAttribute import ConditionalAttribute
from __init___parts.TraceListener import TraceListener
from __init___parts.TextWriterTraceListener import TextWriterTraceListener
from __init___parts.ConsoleTraceListener import ConsoleTraceListener
from __init___parts.CorrelationManager import CorrelationManager
from __init___parts.CounterCreationData import CounterCreationData
from __init___parts.CounterCreationDataCollection import CounterCreationDataCollection
from __init___parts.CounterSample import CounterSample
from __init___parts.CounterSampleCalculator import CounterSampleCalculator
from __init___parts.DataReceivedEventArgs import DataReceivedEventArgs
from __init___parts.DataReceivedEventHandler import DataReceivedEventHandler
from __init___parts.Debug import Debug
from __init___parts.DebuggableAttribute import DebuggableAttribute
from __init___parts.Debugger import Debugger
from __init___parts.DebuggerBrowsableAttribute import DebuggerBrowsableAttribute
from __init___parts.DebuggerBrowsableState import DebuggerBrowsableState
from __init___parts.DebuggerDisplayAttribute import DebuggerDisplayAttribute
from __init___parts.DebuggerHiddenAttribute import DebuggerHiddenAttribute
from __init___parts.DebuggerNonUserCodeAttribute import DebuggerNonUserCodeAttribute
from __init___parts.DebuggerStepperBoundaryAttribute import DebuggerStepperBoundaryAttribute
from __init___parts.DebuggerStepThroughAttribute import DebuggerStepThroughAttribute
from __init___parts.DebuggerTypeProxyAttribute import DebuggerTypeProxyAttribute
from __init___parts.DebuggerVisualizerAttribute import DebuggerVisualizerAttribute
from __init___parts.DefaultTraceListener import DefaultTraceListener
from __init___parts.DelimitedListTraceListener import DelimitedListTraceListener
from __init___parts.DiagnosticsConfigurationHandler import DiagnosticsConfigurationHandler
from __init___parts.EntryWrittenEventArgs import EntryWrittenEventArgs
from __init___parts.EntryWrittenEventHandler import EntryWrittenEventHandler
from __init___parts.EventInstance import EventInstance
from __init___parts.EventLog import EventLog
from __init___parts.EventLogEntry import EventLogEntry
from __init___parts.EventLogEntryCollection import EventLogEntryCollection
from __init___parts.EventLogEntryType import EventLogEntryType
from __init___parts.EventLogPermission import EventLogPermission
from __init___parts.EventLogPermissionAccess import EventLogPermissionAccess
from __init___parts.EventLogPermissionAttribute import EventLogPermissionAttribute
from __init___parts.EventLogPermissionEntry import EventLogPermissionEntry
from __init___parts.EventLogPermissionEntryCollection import EventLogPermissionEntryCollection
from __init___parts.EventLogTraceListener import EventLogTraceListener
from __init___parts.EventSourceCreationData import EventSourceCreationData
from __init___parts.TraceFilter import TraceFilter
from __init___parts.EventTypeFilter import EventTypeFilter
from __init___parts.FileVersionInfo import FileVersionInfo
from __init___parts.ICollectData import ICollectData
from __init___parts.InstanceData import InstanceData
from __init___parts.InstanceDataCollection import InstanceDataCollection
from __init___parts.InstanceDataCollectionCollection import InstanceDataCollectionCollection
from __init___parts.MonitoringDescriptionAttribute import MonitoringDescriptionAttribute
from __init___parts.OverflowAction import OverflowAction
from __init___parts.PerformanceCounter import PerformanceCounter
from __init___parts.PerformanceCounterCategory import PerformanceCounterCategory
from __init___parts.PerformanceCounterCategoryType import PerformanceCounterCategoryType
from __init___parts.PerformanceCounterInstanceLifetime import PerformanceCounterInstanceLifetime
from __init___parts.PerformanceCounterManager import PerformanceCounterManager
from __init___parts.PerformanceCounterPermission import PerformanceCounterPermission
from __init___parts.PerformanceCounterPermissionAccess import PerformanceCounterPermissionAccess
from __init___parts.PerformanceCounterPermissionAttribute import PerformanceCounterPermissionAttribute
from __init___parts.PerformanceCounterPermissionEntry import PerformanceCounterPermissionEntry
from __init___parts.PerformanceCounterPermissionEntryCollection import PerformanceCounterPermissionEntryCollection
from __init___parts.PerformanceCounterType import PerformanceCounterType
from __init___parts.Process import Process
from __init___parts.ProcessModule import ProcessModule
from __init___parts.ProcessModuleCollection import ProcessModuleCollection
from __init___parts.ProcessPriorityClass import ProcessPriorityClass
from __init___parts.ProcessStartInfo import ProcessStartInfo
from __init___parts.ProcessThread import ProcessThread
from __init___parts.ProcessThreadCollection import ProcessThreadCollection
from __init___parts.ProcessWindowStyle import ProcessWindowStyle
from __init___parts.SourceFilter import SourceFilter
from __init___parts.SourceLevels import SourceLevels
from __init___parts.SourceSwitch import SourceSwitch
from __init___parts.StackFrame import StackFrame
from __init___parts.StackTrace import StackTrace
from __init___parts.Stopwatch import Stopwatch
from __init___parts.SwitchAttribute import SwitchAttribute
from __init___parts.SwitchLevelAttribute import SwitchLevelAttribute
from __init___parts.ThreadPriorityLevel import ThreadPriorityLevel
from __init___parts.ThreadState import ThreadState
from __init___parts.ThreadWaitReason import ThreadWaitReason
from __init___parts.Trace import Trace
from __init___parts.TraceEventCache import TraceEventCache
from __init___parts.TraceEventType import TraceEventType
from __init___parts.TraceLevel import TraceLevel
from __init___parts.TraceListenerCollection import TraceListenerCollection
from __init___parts.TraceOptions import TraceOptions
from __init___parts.TraceSource import TraceSource
from __init___parts.TraceSwitch import TraceSwitch
from __init___parts.XmlWriterTraceListener import XmlWriterTraceListener



# encoding: utf-8
# module System.Security.Authentication calls itself Authentication
# from System,Version=4.0.0.0,Culture=neutral,PublicKeyToken=b77a5c561934e089
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class AuthenticationException(SystemException,ISerializable,_Exception):
 """
 The exception that is thrown when authentication fails for an authentication stream.

 

 AuthenticationException()

 AuthenticationException(message: str)

 AuthenticationException(message: str,innerException: Exception)
 """
 def add_SerializeObjectState(self,*args):
  """ add_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def remove_SerializeObjectState(self,*args):
  """ remove_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,message=None,innerException=None):
  """
  __new__(cls: type)

  __new__(cls: type,serializationInfo: SerializationInfo,streamingContext: StreamingContext)

  __new__(cls: type,message: str)

  __new__(cls: type,message: str,innerException: Exception)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass

class CipherAlgorithmType(Enum,IComparable,IFormattable,IConvertible):
 """
 Defines the possible cipher algorithms for the System.Net.Security.SslStream class.

 

 enum CipherAlgorithmType,values: Aes (26129),Aes128 (26126),Aes192 (26127),Aes256 (26128),Des (26113),None (0),Null (24576),Rc2 (26114),Rc4 (26625),TripleDes (26115)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Aes=None
 Aes128=None
 Aes192=None
 Aes256=None
 Des=None
 None=None
 Null=None
 Rc2=None
 Rc4=None
 TripleDes=None
 value__=None


class ExchangeAlgorithmType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the algorithm used to create keys shared by the client and server.

 

 enum ExchangeAlgorithmType,values: DiffieHellman (43522),None (0),RsaKeyX (41984),RsaSign (9216)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 DiffieHellman=None
 None=None
 RsaKeyX=None
 RsaSign=None
 value__=None


class HashAlgorithmType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the algorithm used for generating message authentication codes (MACs).

 

 enum HashAlgorithmType,values: Md5 (32771),None (0),Sha1 (32772),Sha256 (32780),Sha384 (32781),Sha512 (32782)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Md5=None
 None=None
 Sha1=None
 Sha256=None
 Sha384=None
 Sha512=None
 value__=None


class InvalidCredentialException(AuthenticationException,ISerializable,_Exception):
 """
 The exception that is thrown when authentication fails for an authentication stream and cannot be retried.

 

 InvalidCredentialException()

 InvalidCredentialException(message: str)

 InvalidCredentialException(message: str,innerException: Exception)
 """
 def add_SerializeObjectState(self,*args):
  """ add_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def remove_SerializeObjectState(self,*args):
  """ remove_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,message=None,innerException=None):
  """
  __new__(cls: type)

  __new__(cls: type,serializationInfo: SerializationInfo,streamingContext: StreamingContext)

  __new__(cls: type,message: str)

  __new__(cls: type,message: str,innerException: Exception)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass

class SslProtocols(Enum,IComparable,IFormattable,IConvertible):
 """
 Defines the possible versions of System.Security.Authentication.SslProtocols.

 

 enum (flags) SslProtocols,values: Default (240),None (0),Ssl2 (12),Ssl3 (48),Tls (192),Tls11 (768),Tls12 (3072)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Default=None
 None=None
 Ssl2=None
 Ssl3=None
 Tls=None
 Tls11=None
 Tls12=None
 value__=None


# variables with complex values




class BindingMemberInfo(object):
 """
 Contains information that enables a System.Windows.Forms.Binding to resolve a data binding to either the property of an object or the property of the current object in a list of objects.

 

 BindingMemberInfo(dataMember: str)
 """
 def Equals(self,otherObject):
  """
  Equals(self: BindingMemberInfo,otherObject: object) -> bool

  

   Determines whether the specified object is equal to this System.Windows.Forms.BindingMemberInfo.

  

   otherObject: The object to compare for equality.

   Returns: true if otherObject is a System.Windows.Forms.BindingMemberInfo and both 

    System.Windows.Forms.BindingMemberInfo.BindingMember strings are equal; otherwise false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: BindingMemberInfo) -> int

  

   Returns the hash code for this System.Windows.Forms.BindingMemberInfo.

   Returns: The hash code for this System.Windows.Forms.BindingMemberInfo.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 @staticmethod
 def __new__(self,dataMember):
  """
  __new__(cls: type,dataMember: str)

  __new__[BindingMemberInfo]() -> BindingMemberInfo
  """
  pass
 def __ne__(self,*args):
  pass
 BindingField=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the property name of the data-bound object.



Get: BindingField(self: BindingMemberInfo) -> str



"""

 BindingMember=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the information that is used to specify the property name of the data-bound object.



Get: BindingMember(self: BindingMemberInfo) -> str



"""

 BindingPath=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the property name,or the period-delimited hierarchy of property names,that comes before the property name of the data-bound object.



Get: BindingPath(self: BindingMemberInfo) -> str



"""





class WebBrowserProgressChangedEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.WebBrowser.ProgressChanged event.

 

 WebBrowserProgressChangedEventArgs(currentProgress: Int64,maximumProgress: Int64)
 """
 @staticmethod
 def __new__(self,currentProgress,maximumProgress):
  """ __new__(cls: type,currentProgress: Int64,maximumProgress: Int64) """
  pass
 CurrentProgress=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of bytes that have been downloaded.



Get: CurrentProgress(self: WebBrowserProgressChangedEventArgs) -> Int64



"""

 MaximumProgress=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the total number of bytes in the document being loaded.



Get: MaximumProgress(self: WebBrowserProgressChangedEventArgs) -> Int64



"""





class FileVersionInfo(object):
 """ Provides version information for a physical file on disk. """
 @staticmethod
 def GetVersionInfo(fileName):
  """
  GetVersionInfo(fileName: str) -> FileVersionInfo

  

   Returns a System.Diagnostics.FileVersionInfo representing the version information associated 

    with the specified file.

  

  

   fileName: The fully qualified path and name of the file to retrieve the version information for.

   Returns: A System.Diagnostics.FileVersionInfo containing information about the file. If the file did not 

    contain version information,the System.Diagnostics.FileVersionInfo contains only the name of 

    the file requested.
  """
  pass
 def ToString(self):
  """
  ToString(self: FileVersionInfo) -> str

  

   Returns a partial list of properties in the System.Diagnostics.FileVersionInfo and their values.

   Returns: A list of the following properties in this class and their values: 

    System.Diagnostics.FileVersionInfo.FileName,System.Diagnostics.FileVersionInfo.InternalName,

    System.Diagnostics.FileVersionInfo.OriginalFilename,

    System.Diagnostics.FileVersionInfo.FileVersion,

    System.Diagnostics.FileVersionInfo.FileDescription,

    System.Diagnostics.FileVersionInfo.ProductName,

    System.Diagnostics.FileVersionInfo.ProductVersion,System.Diagnostics.FileVersionInfo.IsDebug,

    System.Diagnostics.FileVersionInfo.IsPatched,System.Diagnostics.FileVersionInfo.IsPreRelease,

    System.Diagnostics.FileVersionInfo.IsPrivateBuild,

    System.Diagnostics.FileVersionInfo.IsSpecialBuild,System.Diagnostics.FileVersionInfo.Language.If 

    the file did not contain version information,this list will contain only the name of the 

    requested file. Boolean values will be false,and all other entries will be null.
  """
  pass
 Comments=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the comments associated with the file.



Get: Comments(self: FileVersionInfo) -> str



"""

 CompanyName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the company that produced the file.



Get: CompanyName(self: FileVersionInfo) -> str



"""

 FileBuildPart=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the build number of the file.



Get: FileBuildPart(self: FileVersionInfo) -> int



"""

 FileDescription=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the description of the file.



Get: FileDescription(self: FileVersionInfo) -> str



"""

 FileMajorPart=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the major part of the version number.



Get: FileMajorPart(self: FileVersionInfo) -> int



"""

 FileMinorPart=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the minor part of the version number of the file.



Get: FileMinorPart(self: FileVersionInfo) -> int



"""

 FileName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the file that this instance of System.Diagnostics.FileVersionInfo describes.



Get: FileName(self: FileVersionInfo) -> str



"""

 FilePrivatePart=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the file private part number.



Get: FilePrivatePart(self: FileVersionInfo) -> int



"""

 FileVersion=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the file version number.



Get: FileVersion(self: FileVersionInfo) -> str



"""

 InternalName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the internal name of the file,if one exists.



Get: InternalName(self: FileVersionInfo) -> str



"""

 IsDebug=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that specifies whether the file contains debugging information or is compiled with debugging features enabled.



Get: IsDebug(self: FileVersionInfo) -> bool



"""

 IsPatched=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that specifies whether the file has been modified and is not identical to the original shipping file of the same version number.



Get: IsPatched(self: FileVersionInfo) -> bool



"""

 IsPreRelease=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that specifies whether the file is a development version,rather than a commercially released product.



Get: IsPreRelease(self: FileVersionInfo) -> bool



"""

 IsPrivateBuild=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that specifies whether the file was built using standard release procedures.



Get: IsPrivateBuild(self: FileVersionInfo) -> bool



"""

 IsSpecialBuild=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that specifies whether the file is a special build.



Get: IsSpecialBuild(self: FileVersionInfo) -> bool



"""

 Language=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the default language string for the version info block.



Get: Language(self: FileVersionInfo) -> str



"""

 LegalCopyright=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets all copyright notices that apply to the specified file.



Get: LegalCopyright(self: FileVersionInfo) -> str



"""

 LegalTrademarks=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the trademarks and registered trademarks that apply to the file.



Get: LegalTrademarks(self: FileVersionInfo) -> str



"""

 OriginalFilename=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name the file was created with.



Get: OriginalFilename(self: FileVersionInfo) -> str



"""

 PrivateBuild=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets information about a private version of the file.



Get: PrivateBuild(self: FileVersionInfo) -> str



"""

 ProductBuildPart=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the build number of the product this file is associated with.



Get: ProductBuildPart(self: FileVersionInfo) -> int



"""

 ProductMajorPart=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the major part of the version number for the product this file is associated with.



Get: ProductMajorPart(self: FileVersionInfo) -> int



"""

 ProductMinorPart=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the minor part of the version number for the product the file is associated with.



Get: ProductMinorPart(self: FileVersionInfo) -> int



"""

 ProductName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the product this file is distributed with.



Get: ProductName(self: FileVersionInfo) -> str



"""

 ProductPrivatePart=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the private part number of the product this file is associated with.



Get: ProductPrivatePart(self: FileVersionInfo) -> int



"""

 ProductVersion=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the version of the product this file is distributed with.



Get: ProductVersion(self: FileVersionInfo) -> str



"""

 SpecialBuild=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the special build information for the file.



Get: SpecialBuild(self: FileVersionInfo) -> str



"""





from typing import Tuple, Set, Iterable, List


class LanguageType:
    English_USA = 0
    German = 1
    Spanish = 2
    French = 3
    Italian = 4
    Dutch = 5
    Chinese_Simplified = 6
    Chinese_Traditional = 7
    Japanese = 8
    Korean = 9
    Russian = 10
    Czech = 11
    Polish = 12
    Hungarian = 13
    Brazilian_Portuguese = 14
    English_GB = 15
    Unknown = -1


class ProductType:
    Architecture = 0
    Structure = 1
    MEP = 2
    Revit = 3
    LT = 4
    Unknown = 5


class Application:
    def add_ApplicationInitialized(self, handler: EventHandler) -> None: ...
    def remove_ApplicationInitialized(self, handler: EventHandler) -> None: ...
    def add_DocumentChanged(self, handler: EventHandler) -> None: ...
    def remove_DocumentChanged(self, handler: EventHandler) -> None: ...
    def add_FailuresProcessing(self, handler: EventHandler) -> None: ...
    def remove_FailuresProcessing(self, handler: EventHandler) -> None: ...
    def add_DocumentSaving(self, handler: EventHandler) -> None: ...
    def remove_DocumentSaving(self, handler: EventHandler) -> None: ...
    def add_DocumentSaved(self, handler: EventHandler) -> None: ...
    def remove_DocumentSaved(self, handler: EventHandler) -> None: ...
    def add_DocumentSavingAs(self, handler: EventHandler) -> None: ...
    def remove_DocumentSavingAs(self, handler: EventHandler) -> None: ...
    def add_DocumentSavedAs(self, handler: EventHandler) -> None: ...
    def remove_DocumentSavedAs(self, handler: EventHandler) -> None: ...
    def add_DocumentPrinting(self, handler: EventHandler) -> None: ...
    def remove_DocumentPrinting(self, handler: EventHandler) -> None: ...
    def add_DocumentPrinted(self, handler: EventHandler) -> None: ...
    def remove_DocumentPrinted(self, handler: EventHandler) -> None: ...
    def add_ViewPrinting(self, handler: EventHandler) -> None: ...
    def remove_ViewPrinting(self, handler: EventHandler) -> None: ...
    def add_ViewPrinted(self, handler: EventHandler) -> None: ...
    def remove_ViewPrinted(self, handler: EventHandler) -> None: ...
    def add_DocumentClosing(self, handler: EventHandler) -> None: ...
    def remove_DocumentClosing(self, handler: EventHandler) -> None: ...
    def add_DocumentClosed(self, handler: EventHandler) -> None: ...
    def remove_DocumentClosed(self, handler: EventHandler) -> None: ...
    def add_DocumentCreating(self, handler: EventHandler) -> None: ...
    def remove_DocumentCreating(self, handler: EventHandler) -> None: ...
    def add_DocumentCreated(self, handler: EventHandler) -> None: ...
    def remove_DocumentCreated(self, handler: EventHandler) -> None: ...
    def add_FileImporting(self, handler: EventHandler) -> None: ...
    def remove_FileImporting(self, handler: EventHandler) -> None: ...
    def add_FileImported(self, handler: EventHandler) -> None: ...
    def remove_FileImported(self, handler: EventHandler) -> None: ...
    def add_FileExporting(self, handler: EventHandler) -> None: ...
    def remove_FileExporting(self, handler: EventHandler) -> None: ...
    def add_FileExported(self, handler: EventHandler) -> None: ...
    def remove_FileExported(self, handler: EventHandler) -> None: ...
    def add_ViewExporting(self, handler: EventHandler) -> None: ...
    def remove_ViewExporting(self, handler: EventHandler) -> None: ...
    def add_ViewExported(self, handler: EventHandler) -> None: ...
    def remove_ViewExported(self, handler: EventHandler) -> None: ...
    def add_ViewsExportingByContext(self, handler: EventHandler) -> None: ...
    def remove_ViewsExportingByContext(self, handler: EventHandler) -> None: ...
    def add_ViewsExportedByContext(self, handler: EventHandler) -> None: ...
    def remove_ViewsExportedByContext(self, handler: EventHandler) -> None: ...
    def add_DocumentOpening(self, handler: EventHandler) -> None: ...
    def remove_DocumentOpening(self, handler: EventHandler) -> None: ...
    def add_DocumentOpened(self, handler: EventHandler) -> None: ...
    def remove_DocumentOpened(self, handler: EventHandler) -> None: ...
    def add_DocumentSynchronizingWithCentral(self, handler: EventHandler) -> None: ...
    def remove_DocumentSynchronizingWithCentral(self, handler: EventHandler) -> None: ...
    def add_DocumentSynchronizedWithCentral(self, handler: EventHandler) -> None: ...
    def remove_DocumentSynchronizedWithCentral(self, handler: EventHandler) -> None: ...
    def add_LinkedResourceOpening(self, handler: EventHandler) -> None: ...
    def remove_LinkedResourceOpening(self, handler: EventHandler) -> None: ...
    def add_LinkedResourceOpened(self, handler: EventHandler) -> None: ...
    def remove_LinkedResourceOpened(self, handler: EventHandler) -> None: ...
    def add_WorksharedOperationProgressChanged(self, handler: EventHandler) -> None: ...
    def remove_WorksharedOperationProgressChanged(self, handler: EventHandler) -> None: ...
    def add_ProgressChanged(self, handler: EventHandler) -> None: ...
    def remove_ProgressChanged(self, handler: EventHandler) -> None: ...
    def add_DocumentWorksharingEnabled(self, handler: EventHandler) -> None: ...
    def remove_DocumentWorksharingEnabled(self, handler: EventHandler) -> None: ...
    def add_FamilyLoadingIntoDocument(self, handler: EventHandler) -> None: ...
    def remove_FamilyLoadingIntoDocument(self, handler: EventHandler) -> None: ...
    def add_FamilyLoadedIntoDocument(self, handler: EventHandler) -> None: ...
    def remove_FamilyLoadedIntoDocument(self, handler: EventHandler) -> None: ...
    def add_ElementTypeDuplicating(self, handler: EventHandler) -> None: ...
    def remove_ElementTypeDuplicating(self, handler: EventHandler) -> None: ...
    def add_ElementTypeDuplicated(self, handler: EventHandler) -> None: ...
    def remove_ElementTypeDuplicated(self, handler: EventHandler) -> None: ...
    def add_DocumentReloadingLatest(self, handler: EventHandler) -> None: ...
    def remove_DocumentReloadingLatest(self, handler: EventHandler) -> None: ...
    def add_DocumentReloadedLatest(self, handler: EventHandler) -> None: ...
    def remove_DocumentReloadedLatest(self, handler: EventHandler) -> None: ...
    def Dispose(self) -> None: ...
    @property
    def Documents(self) -> DocumentSet: ...
    @overload
    def NewProjectDocument(self, unitSystem: UnitSystem) -> Document: ...
    @overload
    def NewProjectDocument(self, templateFileName: str) -> Document: ...
    def NewProjectTemplateDocument(self, templateFilename: str) -> Document: ...
    def NewFamilyDocument(self, templateFileName: str) -> Document: ...
    def OpenBuildingComponentDocument(self, fileName: str) -> Document: ...
    def ExtractPartAtomFromFamilyFile(self, familyFilePath: str, xmlFilePath: str) -> None: ...
    @property
    def RecordingJournalFilename(self) -> str: ...
    @property
    def Create(self) -> Application: ...
    def OpenSharedParameterFile(self) -> DefinitionFile: ...
    @property
    def VersionName(self) -> str: ...
    @property
    def VersionNumber(self) -> str: ...
    @property
    def VersionBuild(self) -> str: ...
    @property
    def Cities(self) -> CitySet: ...
    @property
    def Language(self) -> LanguageType: ...
    @property
    def Product(self) -> ProductType: ...
    @property
    def SharedParametersFilename(self) -> str: ...
    @SharedParametersFilename.setter
    def SharedParametersFilename(self, filename: str) -> None: ...
    @property
    def ActiveAddInId(self) -> AddInId: ...
    @property
    def Username(self) -> str: ...
    @property
    def SubVersionNumber(self) -> str: ...
    @property
    def FamilyTemplatePath(self) -> str: ...
    @property
    def PointCloudsRootPath(self) -> str: ...
    @property
    def SystemsAnalysisWorkfilesRootPath(self) -> str: ...
    @property
    def DefaultProjectTemplate(self) -> str: ...
    @property
    def DefaultIFCProjectTemplate(self) -> str: ...
    @property
    def ExportIFCCategoryTable(self) -> str: ...
    @property
    def ImportIFCCategoryTable(self) -> str: ...
    @property
    def ShowGraphicalWarningDuctDisconnects(self) -> bool: ...
    @ShowGraphicalWarningDuctDisconnects.setter
    def ShowGraphicalWarningDuctDisconnects(self, bShow: bool) -> None: ...
    @property
    def ShowGraphicalWarningPipeDisconnects(self) -> bool: ...
    @ShowGraphicalWarningPipeDisconnects.setter
    def ShowGraphicalWarningPipeDisconnects(self, bShow: bool) -> None: ...
    @property
    def ShowGraphicalWarningCableTrayConduitDisconnects(self) -> bool: ...
    @ShowGraphicalWarningCableTrayConduitDisconnects.setter
    def ShowGraphicalWarningCableTrayConduitDisconnects(self, bShow: bool) -> None: ...
    @property
    def ShowGraphicalWarningElectricalDisconnects(self) -> bool: ...
    @ShowGraphicalWarningElectricalDisconnects.setter
    def ShowGraphicalWarningElectricalDisconnects(self, bShow: bool) -> None: ...
    @property
    def ShowGraphicalWarningHangerDisconnects(self) -> bool: ...
    @ShowGraphicalWarningHangerDisconnects.setter
    def ShowGraphicalWarningHangerDisconnects(self, bShow: bool) -> None: ...
    @property
    def DefaultViewDiscipline(self) -> ViewDiscipline: ...
    @DefaultViewDiscipline.setter
    def DefaultViewDiscipline(self, viewDiscipline: ViewDiscipline) -> None: ...
    @property
    def VertexTolerance(self) -> float: ...
    @property
    def AngleTolerance(self) -> float: ...
    @property
    def IsArchitectureEnabled(self) -> bool: ...
    @IsArchitectureEnabled.setter
    def IsArchitectureEnabled(self, bArchitectureEnabled: bool) -> None: ...
    @property
    def IsStructureEnabled(self) -> bool: ...
    @IsStructureEnabled.setter
    def IsStructureEnabled(self, bStructureEnabled: bool) -> None: ...
    @property
    def IsSystemsEnabled(self) -> bool: ...
    @property
    def IsMassingEnabled(self) -> bool: ...
    @IsMassingEnabled.setter
    def IsMassingEnabled(self, bMassingEnabled: bool) -> None: ...
    @property
    def IsEnergyAnalysisEnabled(self) -> bool: ...
    @IsEnergyAnalysisEnabled.setter
    def IsEnergyAnalysisEnabled(self, bEenergyAnalysisEnabled: bool) -> None: ...
    @property
    def IsStructuralAnalysisEnabled(self) -> bool: ...
    @IsStructuralAnalysisEnabled.setter
    def IsStructuralAnalysisEnabled(self, bStructuralAnalysisEnabled: bool) -> None: ...
    @property
    def IsMechanicalEnabled(self) -> bool: ...
    @IsMechanicalEnabled.setter
    def IsMechanicalEnabled(self, bMechanicalEnabled: bool) -> None: ...
    @property
    def IsElectricalEnabled(self) -> bool: ...
    @IsElectricalEnabled.setter
    def IsElectricalEnabled(self, bElectricalEnabled: bool) -> None: ...
    @property
    def IsPipingEnabled(self) -> bool: ...
    @IsPipingEnabled.setter
    def IsPipingEnabled(self, bPipingEnabled: bool) -> None: ...
    @property
    def IsMechanicalAnalysisEnabled(self) -> bool: ...
    @IsMechanicalAnalysisEnabled.setter
    def IsMechanicalAnalysisEnabled(self, bMechanicalAnalysisEnabled: bool) -> None: ...
    @property
    def IsElectricalAnalysisEnabled(self) -> bool: ...
    @IsElectricalAnalysisEnabled.setter
    def IsElectricalAnalysisEnabled(self, bElectricalAnalysisEnabled: bool) -> None: ...
    @property
    def IsPipingAnalysisEnabled(self) -> bool: ...
    @IsPipingAnalysisEnabled.setter
    def IsPipingAnalysisEnabled(self, bPipingAnalysisEnabled: bool) -> None: ...
    @property
    def IsRouteAnalysisEnabled(self) -> bool: ...
    @IsRouteAnalysisEnabled.setter
    def IsRouteAnalysisEnabled(self, bRouteAnalysisEnabled: bool) -> None: ...
    @property
    def IsInfrastructureEnabled(self) -> bool: ...
    @IsInfrastructureEnabled.setter
    def IsInfrastructureEnabled(self, bInfrastructureEnabled: bool) -> None: ...
    @property
    def CurrentRevitServerAccelerator(self) -> str: ...
    @CurrentRevitServerAccelerator.setter
    def CurrentRevitServerAccelerator(self, strAcceleratorName: str) -> None: ...
    @property
    def ShortCurveTolerance(self) -> float: ...
    @property
    def MinimumThickness() -> float: ...
    @property
    def CurrentUserAddinsLocation(self) -> str: ...
    @property
    def AllUsersAddinsLocation(self) -> str: ...
    @property
    def CurrentUsersDataFolderPath(self) -> str: ...
    @property
    def CurrentUsersAddinsDataFolderPath(self) -> str: ...
    @property
    def BackgroundColor(self) -> Color: ...
    @BackgroundColor.setter
    def BackgroundColor(self, backgroundColor: Color) -> None: ...
    @property
    def IsLoggedIn() -> bool: ...
    @property
    def LoginUserId(self) -> str: ...
    @property
    def AllowNavigationDuringRedraw(self) -> bool: ...
    @AllowNavigationDuringRedraw.setter
    def AllowNavigationDuringRedraw(self, allowNavigationDuringRedraw: bool) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    @overload
    def OpenIFCDocument(self, fileName: str, importOptions: IFCImportOptions) -> Document: ...
    @overload
    def OpenIFCDocument(self, fileName: str) -> Document: ...
    def RegisterFailuresProcessor(processor: IFailuresProcessor) -> None: ...
    def GetFailureDefinitionRegistry() -> FailureDefinitionRegistry: ...
    def WriteJournalComment(self, comment: str, timeStamp: bool) -> None: ...
    def IsJournalPlaying(self) -> bool: ...
    def GetSystemsAnalysisWorkflowNames(self) -> List[str]: ...
    def GetSystemsAnalysisWorkflows(self) -> IDictionary: ...
    def SetSystemsAnalysisWorkflows(self, paths: IDictionary) -> None: ...
    def GetLibraryPaths(self) -> IDictionary: ...
    def SetLibraryPaths(self, paths: IDictionary) -> None: ...
    def PurgeReleasedAPIObjects(self) -> None: ...
    def CopyModel(self, sourceModelPath: ModelPath, destFilePath: str, overwrite: bool) -> None: ...
    def GetWorksharingCentralGUID(self, serverModelPath: ServerPath) -> Guid: ...
    def GetRevitServerNetworkHosts(self) -> List[str]: ...
    def IsValidThickness(thickness: float) -> bool: ...
    @overload
    def OpenDocumentFile(self, fileName: str) -> Document: ...
    @overload
    def OpenDocumentFile(self, modelPath: ModelPath, openOptions: OpenOptions, openFromCloudCallback: IOpenFromCloudCallback) -> Document: ...
    @overload
    def OpenDocumentFile(self, modelPath: ModelPath, openOptions: OpenOptions) -> Document: ...
    def UpdateRenderAppearanceLibrary(self) -> None: ...
    def GetAssets(self, assetType: AssetType) -> List[Asset]: ...


class ControlledApplication:
    @property
    def IsLateAddinLoading(self) -> bool: ...
    def GetFailureDefinitionRegistry() -> FailureDefinitionRegistry: ...
    def RegisterFailuresProcessor(processor: IFailuresProcessor) -> None: ...
    @property
    def RecordingJournalFilename(self) -> str: ...
    def WriteJournalComment(self, comment: str, timeStamp: bool) -> None: ...
    def IsJournalPlaying(self) -> bool: ...
    def GetLibraryPaths(self) -> IDictionary: ...
    def SetLibraryPaths(self, paths: IDictionary) -> None: ...
    def add_DocumentChanged(self, handler: EventHandler) -> None: ...
    def remove_DocumentChanged(self, handler: EventHandler) -> None: ...
    def add_LinkedResourceOpening(self, handler: EventHandler) -> None: ...
    def remove_LinkedResourceOpening(self, handler: EventHandler) -> None: ...
    def add_LinkedResourceOpened(self, handler: EventHandler) -> None: ...
    def remove_LinkedResourceOpened(self, handler: EventHandler) -> None: ...
    def add_WorksharedOperationProgressChanged(self, handler: EventHandler) -> None: ...
    def remove_WorksharedOperationProgressChanged(self, handler: EventHandler) -> None: ...
    def add_FailuresProcessing(self, handler: EventHandler) -> None: ...
    def remove_FailuresProcessing(self, handler: EventHandler) -> None: ...
    def add_DocumentSaving(self, handler: EventHandler) -> None: ...
    def remove_DocumentSaving(self, handler: EventHandler) -> None: ...
    def add_DocumentSaved(self, handler: EventHandler) -> None: ...
    def remove_DocumentSaved(self, handler: EventHandler) -> None: ...
    def add_DocumentSavingAs(self, handler: EventHandler) -> None: ...
    def remove_DocumentSavingAs(self, handler: EventHandler) -> None: ...
    def add_DocumentSavedAs(self, handler: EventHandler) -> None: ...
    def remove_DocumentSavedAs(self, handler: EventHandler) -> None: ...
    def add_DocumentPrinting(self, handler: EventHandler) -> None: ...
    def remove_DocumentPrinting(self, handler: EventHandler) -> None: ...
    def add_DocumentPrinted(self, handler: EventHandler) -> None: ...
    def remove_DocumentPrinted(self, handler: EventHandler) -> None: ...
    def add_ViewPrinting(self, handler: EventHandler) -> None: ...
    def remove_ViewPrinting(self, handler: EventHandler) -> None: ...
    def add_ViewPrinted(self, handler: EventHandler) -> None: ...
    def remove_ViewPrinted(self, handler: EventHandler) -> None: ...
    def add_DocumentClosing(self, handler: EventHandler) -> None: ...
    def remove_DocumentClosing(self, handler: EventHandler) -> None: ...
    def add_DocumentClosed(self, handler: EventHandler) -> None: ...
    def remove_DocumentClosed(self, handler: EventHandler) -> None: ...
    def add_DocumentCreating(self, handler: EventHandler) -> None: ...
    def remove_DocumentCreating(self, handler: EventHandler) -> None: ...
    def add_DocumentCreated(self, handler: EventHandler) -> None: ...
    def remove_DocumentCreated(self, handler: EventHandler) -> None: ...
    def add_FileImporting(self, handler: EventHandler) -> None: ...
    def remove_FileImporting(self, handler: EventHandler) -> None: ...
    def add_FileImported(self, handler: EventHandler) -> None: ...
    def remove_FileImported(self, handler: EventHandler) -> None: ...
    def add_FileExporting(self, handler: EventHandler) -> None: ...
    def remove_FileExporting(self, handler: EventHandler) -> None: ...
    def add_FileExported(self, handler: EventHandler) -> None: ...
    def remove_FileExported(self, handler: EventHandler) -> None: ...
    def add_DocumentOpening(self, handler: EventHandler) -> None: ...
    def remove_DocumentOpening(self, handler: EventHandler) -> None: ...
    def add_DocumentOpened(self, handler: EventHandler) -> None: ...
    def remove_DocumentOpened(self, handler: EventHandler) -> None: ...
    def add_DocumentSynchronizingWithCentral(self, handler: EventHandler) -> None: ...
    def remove_DocumentSynchronizingWithCentral(self, handler: EventHandler) -> None: ...
    def add_DocumentSynchronizedWithCentral(self, handler: EventHandler) -> None: ...
    def remove_DocumentSynchronizedWithCentral(self, handler: EventHandler) -> None: ...
    def add_ProgressChanged(self, handler: EventHandler) -> None: ...
    def remove_ProgressChanged(self, handler: EventHandler) -> None: ...
    def add_ApplicationInitialized(self, handler: EventHandler) -> None: ...
    def remove_ApplicationInitialized(self, handler: EventHandler) -> None: ...
    def add_FamilyLoadingIntoDocument(self, handler: EventHandler) -> None: ...
    def remove_FamilyLoadingIntoDocument(self, handler: EventHandler) -> None: ...
    def add_FamilyLoadedIntoDocument(self, handler: EventHandler) -> None: ...
    def remove_FamilyLoadedIntoDocument(self, handler: EventHandler) -> None: ...
    def add_ElementTypeDuplicating(self, handler: EventHandler) -> None: ...
    def remove_ElementTypeDuplicating(self, handler: EventHandler) -> None: ...
    def add_ElementTypeDuplicated(self, handler: EventHandler) -> None: ...
    def remove_ElementTypeDuplicated(self, handler: EventHandler) -> None: ...
    def add_DocumentReloadingLatest(self, handler: EventHandler) -> None: ...
    def remove_DocumentReloadingLatest(self, handler: EventHandler) -> None: ...
    def add_DocumentReloadedLatest(self, handler: EventHandler) -> None: ...
    def remove_DocumentReloadedLatest(self, handler: EventHandler) -> None: ...
    @property
    def CurrentUserAddinsLocation(self) -> str: ...
    @property
    def AllUsersAddinsLocation(self) -> str: ...
    @property
    def CurrentUsersDataFolderPath(self) -> str: ...
    @property
    def CurrentUsersAddinsDataFolderPath(self) -> str: ...
    @property
    def SubVersionNumber(self) -> str: ...
    @property
    def Create(self) -> Application: ...
    def OpenSharedParameterFile(self) -> DefinitionFile: ...
    @property
    def VersionName(self) -> str: ...
    @property
    def VersionNumber(self) -> str: ...
    @property
    def VersionBuild(self) -> str: ...
    @property
    def Cities(self) -> CitySet: ...
    @property
    def Language(self) -> LanguageType: ...
    @property
    def Product(self) -> ProductType: ...
    @property
    def SharedParametersFilename(self) -> str: ...
    @SharedParametersFilename.setter
    def SharedParametersFilename(self, filename: str) -> None: ...
    @property
    def ActiveAddInId(self) -> AddInId: ...



class ConditionalAttribute(Attribute,_Attribute):
 """
 Indicates to compilers that a method call or attribute should be ignored unless a specified conditional compilation symbol is defined.

 

 ConditionalAttribute(conditionString: str)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,conditionString):
  """ __new__(cls: type,conditionString: str) """
  pass
 def __reduce_ex__(self,*args):
  pass
 ConditionString=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the conditional compilation symbol that is associated with the System.Diagnostics.ConditionalAttribute attribute.



Get: ConditionString(self: ConditionalAttribute) -> str



"""





class SolidBrush(Brush,ICloneable,IDisposable,ISystemColorTracker):
 """
 Defines a brush of a single color. Brushes are used to fill graphics shapes,such as rectangles,ellipses,pies,polygons,and paths. This class cannot be inherited.

 

 SolidBrush(color: Color)
 """
 def Clone(self):
  """
  Clone(self: SolidBrush) -> object

  

   Creates an exact copy of this System.Drawing.SolidBrush object.

   Returns: The System.Drawing.SolidBrush object that this method creates.
  """
  pass
 def Dispose(self):
  """ Dispose(self: SolidBrush,disposing: bool) """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def SetNativeBrush(self,*args):
  """
  SetNativeBrush(self: Brush,brush: IntPtr)

   In a derived class,sets a reference to a GDI+ brush object.

  

   brush: A pointer to the GDI+ brush object.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,color):
  """ __new__(cls: type,color: Color) """
  pass
 Color=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the color of this System.Drawing.SolidBrush object.



Get: Color(self: SolidBrush) -> Color



Set: Color(self: SolidBrush)=value

"""





class UCOMIEnumConnectionPoints:
 """ Use System.Runtime.InteropServices.ComTypes.IEnumConnectionPoints instead. """
 def Clone(self,ppenum):
  """
  Clone(self: UCOMIEnumConnectionPoints) -> UCOMIEnumConnectionPoints

  

   Creates another enumerator that contains the same enumeration state as the current one.
  """
  pass
 def Next(self,celt,rgelt,pceltFetched):
  """
  Next(self: UCOMIEnumConnectionPoints,celt: int) -> (int,Array[UCOMIConnectionPoint],int)

  

   Retrieves a specified number of items in the enumeration sequence.

  

   celt: The number of IConnectionPoint references to return in rgelt.

   Returns: S_OK if the pceltFetched parameter equals the celt parameter; otherwise,S_FALSE.
  """
  pass
 def Reset(self):
  """
  Reset(self: UCOMIEnumConnectionPoints) -> int

  

   Resets the enumeration sequence to the beginning.

   Returns: An HRESULT with the value S_OK.
  """
  pass
 def Skip(self,celt):
  """
  Skip(self: UCOMIEnumConnectionPoints,celt: int) -> int

  

   Skips over a specified number of items in the enumeration sequence.

  

   celt: The number of elements to skip in the enumeration.

   Returns: S_OK if the number of elements skipped equals the celt parameter; otherwise,S_FALSE.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class DefaultTraceListener(TraceListener,IDisposable):
 """
 Provides the default output methods and behavior for tracing.

 

 DefaultTraceListener()
 """
 def Dispose(self):
  """
  Dispose(self: TraceListener,disposing: bool)

   Releases the unmanaged resources used by the System.Diagnostics.TraceListener and optionally 

    releases the managed resources.

  

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def Fail(self,message,detailMessage=None):
  """
  Fail(self: DefaultTraceListener,message: str,detailMessage: str)

   Emits or displays detailed messages and a stack trace for an assertion that always fails.

  

   message: The message to emit or display.

   detailMessage: The detailed message to emit or display.

  Fail(self: DefaultTraceListener,message: str)

   Emits or displays a message and a stack trace for an assertion that always fails.

  

   message: The message to emit or display.
  """
  pass
 def GetSupportedAttributes(self,*args):
  """
  GetSupportedAttributes(self: TraceListener) -> Array[str]

  

   Gets the custom attributes supported by the trace listener.

   Returns: A string array naming the custom attributes supported by the trace listener,or null if there 

    are no custom attributes.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def Write(self,*__args):
  """
  Write(self: DefaultTraceListener,message: str)

   Writes the output to the OutputDebugString function and to the 

    System.Diagnostics.Debugger.Log(System.Int32,System.String,System.String) method.

  

  

   message: The message to write to OutputDebugString and 

    System.Diagnostics.Debugger.Log(System.Int32,System.String,System.String).
  """
  pass
 def WriteIndent(self,*args):
  """
  WriteIndent(self: TraceListener)

   Writes the indent to the listener you create when you implement this class,and resets the 

    System.Diagnostics.TraceListener.NeedIndent property to false.
  """
  pass
 def WriteLine(self,*__args):
  """
  WriteLine(self: DefaultTraceListener,message: str)

   Writes the output to the OutputDebugString function and to the 

    System.Diagnostics.Debugger.Log(System.Int32,System.String,System.String) method,followed by a 

    carriage return and line feed (\r\n).

  

  

   message: The message to write to OutputDebugString and 

    System.Diagnostics.Debugger.Log(System.Int32,System.String,System.String).
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 AssertUiEnabled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the application is running in user-interface mode.



Get: AssertUiEnabled(self: DefaultTraceListener) -> bool



Set: AssertUiEnabled(self: DefaultTraceListener)=value

"""

 LogFileName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of a log file to write trace or debug messages to.



Get: LogFileName(self: DefaultTraceListener) -> str



Set: LogFileName(self: DefaultTraceListener)=value

"""

 NeedIndent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether to indent the output.



"""





class Progress(object,IProgress[T]):
 """
 Progress[T]()

 Progress[T](handler: Action[T])
 """
 def OnReport(self,*args):
  """ OnReport(self: Progress[T],value: T) """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,handler=None):
  """
  __new__(cls: type)

  __new__(cls: type,handler: Action[T])
  """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 ProgressChanged=None




class ComAliasNameAttribute(Attribute,_Attribute):
 """
 Indicates the COM alias for a parameter or field type.

 

 ComAliasNameAttribute(alias: str)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,alias):
  """ __new__(cls: type,alias: str) """
  pass
 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the alias for the field or parameter as found in the type library when it was imported.



Get: Value(self: ComAliasNameAttribute) -> str



"""





class AuthorizationRule(object):
 """ Determines access to securable objects. The derived classes System.Security.AccessControl.AccessRule and System.Security.AccessControl.AuditRule offer specializations for access and audit functionality. """
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """ __new__(cls: type,identity: IdentityReference,accessMask: int,isInherited: bool,inheritanceFlags: InheritanceFlags,propagationFlags: PropagationFlags) """
  pass
 AccessMask=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the access mask for this rule.



"""

 IdentityReference=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Security.Principal.IdentityReference to which this rule applies.



Get: IdentityReference(self: AuthorizationRule) -> IdentityReference



"""

 InheritanceFlags=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the value of flags that determine how this rule is inherited by child objects.



Get: InheritanceFlags(self: AuthorizationRule) -> InheritanceFlags



"""

 IsInherited=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether this rule is explicitly set or is inherited from a parent container object.



Get: IsInherited(self: AuthorizationRule) -> bool



"""

 PropagationFlags=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the value of the propagation flags,which determine how inheritance of this rule is propagated to child objects. This property is significant only when the value of the System.Security.AccessControl.InheritanceFlags enumeration is not System.Security.AccessControl.InheritanceFlags.None.



Get: PropagationFlags(self: AuthorizationRule) -> PropagationFlags



"""





class Menu(Component,IComponent,IDisposable):
 """ Represents the base functionality for all menus. Although System.Windows.Forms.ToolStripDropDown and System.Windows.Forms.ToolStripDropDownMenu replace and add functionality to the System.Windows.Forms.Menu control of previous versions,System.Windows.Forms.Menu is retained for both backward compatibility and future use if you choose. """
 def CloneMenu(self,*args):
  """
  CloneMenu(self: Menu,menuSrc: Menu)

   Copies the System.Windows.Forms.Menu that is passed as a parameter to the current 

    System.Windows.Forms.Menu.

  

  

   menuSrc: The System.Windows.Forms.Menu to copy.
  """
  pass
 def CreateMenuHandle(self,*args):
  """
  CreateMenuHandle(self: Menu) -> IntPtr

  

   Creates a new handle to the System.Windows.Forms.Menu.

   Returns: A handle to the menu if the method succeeds; otherwise,null.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: Menu,disposing: bool)

   Disposes of the resources,other than memory,used by the System.Windows.Forms.Menu.

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def FindMenuItem(self,type,value):
  """
  FindMenuItem(self: Menu,type: int,value: IntPtr) -> MenuItem

  

   Gets the System.Windows.Forms.MenuItem that contains the value specified.

  

   type: The type of item to use to find the System.Windows.Forms.MenuItem.

   value: The item to use to find the System.Windows.Forms.MenuItem.

   Returns: The System.Windows.Forms.MenuItem that matches value; otherwise,null.
  """
  pass
 def FindMergePosition(self,*args):
  """
  FindMergePosition(self: Menu,mergeOrder: int) -> int

  

   Returns the position at which a menu item should be inserted into the menu.

  

   mergeOrder: The merge order position for the menu item to be merged.

   Returns: The position at which a menu item should be inserted into the menu.
  """
  pass
 def GetContextMenu(self):
  """
  GetContextMenu(self: Menu) -> ContextMenu

  

   Gets the System.Windows.Forms.ContextMenu that contains this menu.

   Returns: The System.Windows.Forms.ContextMenu that contains this menu. The default is null.
  """
  pass
 def GetMainMenu(self):
  """
  GetMainMenu(self: Menu) -> MainMenu

  

   Gets the System.Windows.Forms.MainMenu that contains this menu.

   Returns: The System.Windows.Forms.MainMenu that contains this menu.
  """
  pass
 def GetService(self,*args):
  """
  GetService(self: Component,service: Type) -> object

  

   Returns an object that represents a service provided by the System.ComponentModel.Component or 

    by its System.ComponentModel.Container.

  

  

   service: A service provided by the System.ComponentModel.Component.

   Returns: An System.Object that represents a service provided by the System.ComponentModel.Component,or 

    null if the System.ComponentModel.Component does not provide the specified service.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def MergeMenu(self,menuSrc):
  """
  MergeMenu(self: Menu,menuSrc: Menu)

   Merges the System.Windows.Forms.MenuItem objects of one menu with the current menu.

  

   menuSrc: The System.Windows.Forms.Menu whose menu items are merged with the menu items of the current 

    menu.
  """
  pass
 def ProcessCmdKey(self,*args):
  """
  ProcessCmdKey(self: Menu,msg: Message,keyData: Keys) -> (bool,Message)

  

   Processes a command key.

  

   msg: A System.Windows.Forms.Message,passed by reference that represents the window message to 

    process.

  

   keyData: One of the System.Windows.Forms.Keys values that represents the key to process.

   Returns: true if the character was processed by the control; otherwise,false.
  """
  pass
 def ToString(self):
  """
  ToString(self: Menu) -> str

  

   Returns a System.String that represents the System.Windows.Forms.Menu control.

   Returns: A System.String that represents the current System.Windows.Forms.Menu.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """ __new__(cls: type,items: Array[MenuItem]) """
  pass
 def __str__(self,*args):
  pass
 CanRaiseEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the component can raise an event.



"""

 DesignMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the System.ComponentModel.Component is currently in design mode.



"""

 Events=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of event handlers that are attached to this System.ComponentModel.Component.



"""

 Handle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value representing the window handle for the menu.



Get: Handle(self: Menu) -> IntPtr



"""

 IsParent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether this menu contains any menu items. This property is read-only.



Get: IsParent(self: Menu) -> bool



"""

 MdiListItem=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating the System.Windows.Forms.MenuItem that is used to display a list of multiple document interface (MDI) child forms.



Get: MdiListItem(self: Menu) -> MenuItem



"""

 MenuItems=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating the collection of System.Windows.Forms.MenuItem objects associated with the menu.



Get: MenuItems(self: Menu) -> MenuItemCollection



"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of the System.Windows.Forms.Menu.



Get: Name(self: Menu) -> str



Set: Name(self: Menu)=value

"""

 Tag=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets user-defined data associated with the control.



Get: Tag(self: Menu) -> object



Set: Tag(self: Menu)=value

"""


 FindHandle=0
 FindShortcut=1
 MenuItemCollection=None




class FileSystemWatcher(Component,IComponent,IDisposable,ISupportInitialize):
 """
 Listens to the file system change notifications and raises events when a directory,or file in a directory,changes.

 

 FileSystemWatcher()

 FileSystemWatcher(path: str)

 FileSystemWatcher(path: str,filter: str)
 """
 def BeginInit(self):
  """
  BeginInit(self: FileSystemWatcher)

   Begins the initialization of a System.IO.FileSystemWatcher used on a form or used by another 

    component. The initialization occurs at run time.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: FileSystemWatcher,disposing: bool)

   Releases the unmanaged resources used by the System.IO.FileSystemWatcher and optionally releases 

    the managed resources.

  

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def EndInit(self):
  """
  EndInit(self: FileSystemWatcher)

   Ends the initialization of a System.IO.FileSystemWatcher used on a form or used by another 

    component. The initialization occurs at run time.
  """
  pass
 def GetService(self,*args):
  """
  GetService(self: Component,service: Type) -> object

  

   Returns an object that represents a service provided by the System.ComponentModel.Component or 

    by its System.ComponentModel.Container.

  

  

   service: A service provided by the System.ComponentModel.Component.

   Returns: An System.Object that represents a service provided by the System.ComponentModel.Component,or 

    null if the System.ComponentModel.Component does not provide the specified service.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def OnChanged(self,*args):
  """
  OnChanged(self: FileSystemWatcher,e: FileSystemEventArgs)

   Raises the System.IO.FileSystemWatcher.Changed event.

  

   e: A System.IO.FileSystemEventArgs that contains the event data.
  """
  pass
 def OnCreated(self,*args):
  """
  OnCreated(self: FileSystemWatcher,e: FileSystemEventArgs)

   Raises the System.IO.FileSystemWatcher.Created event.

  

   e: A System.IO.FileSystemEventArgs that contains the event data.
  """
  pass
 def OnDeleted(self,*args):
  """
  OnDeleted(self: FileSystemWatcher,e: FileSystemEventArgs)

   Raises the System.IO.FileSystemWatcher.Deleted event.

  

   e: A System.IO.FileSystemEventArgs that contains the event data.
  """
  pass
 def OnError(self,*args):
  """
  OnError(self: FileSystemWatcher,e: ErrorEventArgs)

   Raises the System.IO.FileSystemWatcher.Error event.

  

   e: An System.IO.ErrorEventArgs that contains the event data.
  """
  pass
 def OnRenamed(self,*args):
  """
  OnRenamed(self: FileSystemWatcher,e: RenamedEventArgs)

   Raises the System.IO.FileSystemWatcher.Renamed event.

  

   e: A System.IO.RenamedEventArgs that contains the event data.
  """
  pass
 def WaitForChanged(self,changeType,timeout=None):
  """
  WaitForChanged(self: FileSystemWatcher,changeType: WatcherChangeTypes,timeout: int) -> WaitForChangedResult

  

   A synchronous method that returns a structure that contains specific information on the change 

    that occurred,given the type of change you want to monitor and the time (in milliseconds) to 

    wait before timing out.

  

  

   changeType: The System.IO.WatcherChangeTypes to watch for.

   timeout: The time (in milliseconds) to wait before timing out.

   Returns: A System.IO.WaitForChangedResult that contains specific information on the change that occurred.

  WaitForChanged(self: FileSystemWatcher,changeType: WatcherChangeTypes) -> WaitForChangedResult

  

   A synchronous method that returns a structure that contains specific information on the change 

    that occurred,given the type of change you want to monitor.

  

  

   changeType: The System.IO.WatcherChangeTypes to watch for.

   Returns: A System.IO.WaitForChangedResult that contains specific information on the change that occurred.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,path=None,filter=None):
  """
  __new__(cls: type)

  __new__(cls: type,path: str)

  __new__(cls: type,path: str,filter: str)
  """
  pass
 def __str__(self,*args):
  pass
 CanRaiseEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the component can raise an event.



"""

 DesignMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the System.ComponentModel.Component is currently in design mode.



"""

 EnableRaisingEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the component is enabled.



Get: EnableRaisingEvents(self: FileSystemWatcher) -> bool



Set: EnableRaisingEvents(self: FileSystemWatcher)=value

"""

 Events=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of event handlers that are attached to this System.ComponentModel.Component.



"""

 Filter=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the filter string used to determine what files are monitored in a directory.



Get: Filter(self: FileSystemWatcher) -> str



Set: Filter(self: FileSystemWatcher)=value

"""

 IncludeSubdirectories=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether subdirectories within the specified path should be monitored.



Get: IncludeSubdirectories(self: FileSystemWatcher) -> bool



Set: IncludeSubdirectories(self: FileSystemWatcher)=value

"""

 InternalBufferSize=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the size (in bytes) of the internal buffer.



Get: InternalBufferSize(self: FileSystemWatcher) -> int



Set: InternalBufferSize(self: FileSystemWatcher)=value

"""

 NotifyFilter=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the type of changes to watch for.



Get: NotifyFilter(self: FileSystemWatcher) -> NotifyFilters



Set: NotifyFilter(self: FileSystemWatcher)=value

"""

 Path=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the path of the directory to watch.



Get: Path(self: FileSystemWatcher) -> str



Set: Path(self: FileSystemWatcher)=value

"""

 Site=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets an System.ComponentModel.ISite for the System.IO.FileSystemWatcher.



Get: Site(self: FileSystemWatcher) -> ISite



Set: Site(self: FileSystemWatcher)=value

"""

 SynchronizingObject=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the object used to marshal the event handler calls issued as a result of a directory change.



Get: SynchronizingObject(self: FileSystemWatcher) -> ISynchronizeInvoke



Set: SynchronizingObject(self: FileSystemWatcher)=value

"""


 Changed=None
 Created=None
 Deleted=None
 Error=None
 Renamed=None




class EditorBrowsableAttribute(Attribute,_Attribute):
 """
 Specifies that a property or method is viewable in an editor. This class cannot be inherited.

 

 EditorBrowsableAttribute(state: EditorBrowsableState)

 EditorBrowsableAttribute()
 """
 def Equals(self,obj):
  """
  Equals(self: EditorBrowsableAttribute,obj: object) -> bool

  

   Returns whether the value of the given object is equal to the current 

    System.ComponentModel.EditorBrowsableAttribute.

  

  

   obj: The object to test the value equality of.

   Returns: true if the value of the given object is equal to that of the current; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """ GetHashCode(self: EditorBrowsableAttribute) -> int """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,state=None):
  """
  __new__(cls: type,state: EditorBrowsableState)

  __new__(cls: type)
  """
  pass
 def __ne__(self,*args):
  pass
 State=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the browsable state of the property or method.



Get: State(self: EditorBrowsableAttribute) -> EditorBrowsableState



"""





class DoubleConverter(BaseNumberConverter):
 """
 Provides a type converter to convert double-precision,floating point number objects to and from various other representations.

 

 DoubleConverter()
 """



class RepeatBehavior(object,IFormattable):
 """
 Describes how a System.Windows.Media.Animation.Timeline repeats its simple duration.

 

 RepeatBehavior(count: float)

 RepeatBehavior(duration: TimeSpan)
 """
 def Equals(self,*__args):
  """
  Equals(repeatBehavior1: RepeatBehavior,repeatBehavior2: RepeatBehavior) -> bool

  

   Indicates whether the two specified System.Windows.Media.Animation.RepeatBehavior structures are 

    equal.

  

  

   repeatBehavior1: The first value to compare.

   repeatBehavior2: The second value to compare.

   Returns: true if both the type and repeat behavior of repeatBehavior1 are equal to that of 

    repeatBehavior2; otherwise,false.

  

  Equals(self: RepeatBehavior,repeatBehavior: RepeatBehavior) -> bool

  

   Returns a value that indicates whether this instance is equal to the specified 

    System.Windows.Media.Animation.RepeatBehavior.

  

  

   repeatBehavior: The value to compare to this instance.

   Returns: true if both the type and repeat behavior of repeatBehavior are equal to this instance; 

    otherwise,false.

  

  Equals(self: RepeatBehavior,value: object) -> bool

  

   Indicates whether this instance is equal to the specified object.

  

   value: The object to compare with this instance.

   Returns: true if value is a System.Windows.Media.Animation.RepeatBehavior that represents the same repeat 

    behavior as this instance; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: RepeatBehavior) -> int

  

   Returns the hash code of this instance.

   Returns: A 32-bit signed integer hash code.
  """
  pass
 def ToString(self,formatProvider=None):
  """
  ToString(self: RepeatBehavior,formatProvider: IFormatProvider) -> str

  

   Returns a string representation of this System.Windows.Media.Animation.RepeatBehavior instance 

    with the specified format.

  

  

   formatProvider: The format used to construct the return value.

   Returns: A string representation of this instance.

  ToString(self: RepeatBehavior) -> str

  

   Returns a string representation of this System.Windows.Media.Animation.RepeatBehavior instance.

   Returns: A string representation of this instance.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,count: float)

  __new__(cls: type,duration: TimeSpan)

  __new__[RepeatBehavior]() -> RepeatBehavior
  """
  pass
 def __ne__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of times a System.Windows.Media.Animation.Timeline should repeat.



Get: Count(self: RepeatBehavior) -> float



"""

 Duration=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the total length of time a System.Windows.Media.Animation.Timeline should play.



Get: Duration(self: RepeatBehavior) -> TimeSpan



"""

 HasCount=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the repeat behavior has a specified iteration count.



Get: HasCount(self: RepeatBehavior) -> bool



"""

 HasDuration=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the repeat behavior has a specified repeat duration.



Get: HasDuration(self: RepeatBehavior) -> bool



"""


 Forever=None




class ImageIndexConverter(Int32Converter):
 """
 Provides a type converter to convert data for an image index to and from a string.

 

 ImageIndexConverter()
 """
 def ConvertFrom(self,*__args):
  """
  ConvertFrom(self: ImageIndexConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> object

  

   Converts the specified value object to a 32-bit signed integer object.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: A System.Globalization.CultureInfo to provide locale information.

   value: The System.Object to convert.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def ConvertTo(self,*__args):
  """
  ConvertTo(self: ImageIndexConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object,destinationType: Type) -> object

  

   Converts the specified object to the specified destination type.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context,which can be 

    used to extract additional information about the environment this type converter is being 

    invoked from. This parameter or properties of this parameter can be null.

  

   culture: A System.Globalization.CultureInfo that provides locale information.

   value: The object to convert,typically an index represented as an System.Int32.

   destinationType: The type to convert the object to,often a System.String.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def GetStandardValues(self,context=None):
  """
  GetStandardValues(self: ImageIndexConverter,context: ITypeDescriptorContext) -> StandardValuesCollection

  

   Returns a collection of standard index values for the image list associated with the specified 

    format context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context,which can be 

    used to extract additional information about the environment this type converter is being 

    invoked from. This parameter or properties of this parameter can be null.

  

   Returns: A System.ComponentModel.TypeConverter.StandardValuesCollection that holds a standard set of 

    valid index values. If no image list is found,this collection will contain a single object with 

    a value of -1.
  """
  pass
 def GetStandardValuesExclusive(self,context=None):
  """
  GetStandardValuesExclusive(self: ImageIndexConverter,context: ITypeDescriptorContext) -> bool

  

   Determines if the list of standard values returned from the 

    erload:System.Windows.Forms.ImageIndexConverter.GetStandardValues method is an exclusive list.

  

  

   context: A formatter context.

   Returns: true if the erload:System.Windows.Forms.ImageIndexConverter.GetStandardValues method returns an 

    exclusive list of valid values; otherwise,false. This implementation always returns false.
  """
  pass
 def GetStandardValuesSupported(self,context=None):
  """
  GetStandardValuesSupported(self: ImageIndexConverter,context: ITypeDescriptorContext) -> bool

  

   Determines if the type converter supports a standard set of values that can be picked from a 

    list.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context,which can be 

    used to extract additional information about the environment this type converter is being 

    invoked from. This parameter or properties of this parameter can be null.

  

   Returns: true if the erload:System.Windows.Forms.ImageIndexConverter.GetStandardValues method returns a 

    standard set of values; otherwise,false. Always returns true.
  """
  pass
 IncludeNoneAsStandardValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether a none or null value is valid in the System.ComponentModel.TypeConverter.StandardValuesCollection collection.



"""


